import QtQuick 2.4
import "../qml"

/* Data Table Element
* Show a series of ListModels as columns of data in a Table
*/

Item {
    id: root

    /* list of ListModels, one for each column of the table
    // properties may be attached to each model (as shown in demoModel below), to control the following:
    //      heading - text for header
    // The "label" role of each ListElement is used as the label data in each cell in the column.
    // The "value" role of each ListElement is used as the value data in each cell in the column.
    // The "isDisplayed" role of each ListElement is used to display or hide each cell in the column.
    // Each model in the list is another column in the table.
    */

    // REQUIRED property: for translation to work (via qsTranslate), it must know where the external model is stored.
    // Without this you will likely see blank strings in your data table.
    property string sourceFile: ""

    property alias dataDisplayModel: selectDataModel
    property bool selectable: false

    property string ruleSourceName: "Components/dotted_line_separator_single"

    // default column width, used if column model has no defined width// Width of each separator
    property real separatorWidth: 3 * main.hScaleFactor
    property int colWidth: width / models.length  - separatorWidth
    property int headerHeight: height * 0.07
    property int hAlign: Text.AlignHCenter

    property string colorSelected: main.colorUIBorder
    property string distanceLabel: workoutScreen.unitState === "ENGLISH" ? qsTr("MILES") : qsTr("KM")
    property string paceLabel: workoutScreen.unitState === "ENGLISH" ? qsTr("PACE/MILE") : qsTr("PACE/KM")
    property string unitsLabel: main.isStepmill ? qsTr("SPM") : (workoutScreen.unitState === "ENGLISH" ? qsTr("MPH") : qsTr("KPH"))
    property string distanceLabelTranslated: qsTr(distanceLabel) + application.emptyString
    property string paceLabelTranslated: qsTr(paceLabel) + application.emptyString
    property string unitsLabelTranslated: qsTr(unitsLabel) + application.emptyString

    property bool wasHRAutoSelected: false

    // re-translate the strings when the language changes
    Connections {
        target: application
        onLanguageChanged: {
           root.distanceLabelTranslated = qsTr(root.distanceLabel) + application.emptyString
            root.paceLabelTranslated = qsTr(root.paceLabel) + application.emptyString
            root.unitsLabelTranslated = qsTr(root.unitsLabel) + application.emptyString
        }
    }

    Connections {
        target: workoutScreen
        onUnitStateChanged: {
            root.distanceLabel = workoutScreen.unitState === "ENGLISH" ? qsTr("MILES") : qsTr("KM")
            root.paceLabel = workoutScreen.unitState === "ENGLISH" ? qsTr("PACE/MILE") : qsTr("PACE/KM")
            root.unitsLabel = workoutScreen.unitState === "ENGLISH" ? qsTr("MPH") : qsTr("KPH")
            root.distanceLabelTranslated = qsTr(root.distanceLabel)
            root.paceLabelTranslated = qsTr(root.paceLabel)
            root.unitsLabelTranslated = qsTr(root.unitsLabel)
        }
    }

    Connections {
        target: heartRateManager
        onHeartRateAvailableChanged: {
            if(heartRateManager.heartRateAvailable) {
                if(root.models[4].get(0).isSelected === false) {
                    if(root.selectedCount === 6) {
                        // Drop last entry to make room for heart rate
                        root.removeLastEntry()
                    }
                    root.toggleSelection(4,0,!heartRateManager.heartRateAvailable) // Turn on Heart Rate
                    root.wasHRAutoSelected = true
                }
            } else if(root.models[4].get(0).isSelected === true && root.wasHRAutoSelected ) {
                    root.toggleSelection(4,0,!heartRateManager.heartRateAvailable) // Turn off Heart Rate
            }
        }
    }

    property list<ListModel> models
    property int selectedCount: 0

    onSelectedCountChanged: {
        if(selectedCount === 0) {
            resetDefaultSelection()
        }

        updateDataDisplay()
    }

    function reset() {
        root.wasHRAutoSelected = false
        resetDefaultSelection();
        updateDataDisplay();
    }

    function toggleSelection(colIndex, index, currentIsSelected) {
        if(selectedCount === 0 && currentIsSelected === true) {
            return;
        }

        if(selectedCount === 6 && currentIsSelected === false) {
            return;
        }
        root.models[colIndex].setProperty(index, "isSelected", !currentIsSelected)

        if(root.models[colIndex].get(index).isSelected === true) {
            selectedCount++;
        } else {
            selectedCount--;
        }
    }

    function resetDefaultSelection() {
        if (!selectable)
            return

        var c, i;

        // Reset all selection toggles
        for (c = 0; c < root.models.length; c++) {
            for(i = 0; i < root.models[c].count; i++) {
                root.models[c].setProperty(i, "isSelected", false);
            }
        }

        // Set default selection toggles
        root.models[1].setProperty(0, "isSelected", true); // Time Elapsed
        root.models[2].setProperty(0, "isSelected", true); // Calories
        selectedCount = 3;

        if(root.models[0].get(0).isDisplayed)
            root.models[0].setProperty(0, "isSelected", true); // Floors Climbed
        else if(root.models[0].get(2).isDisplayed)
            root.models[0].setProperty(2, "isSelected", true); // Distance
        else if(root.models[0].get(3).isDisplayed)
            root.models[0].setProperty(3, "isSelected", true); // Distance Remaining
        else if(root.models[0].get(4).isDisplayed)
            root.models[0].setProperty(4, "isSelected", true); // Distance Climbed
        else // No distances displayed
            selectedCount--;

        if (heartRateManager.heartRateAvailable) {
            root.models[4].setProperty(0, "isSelected", true); // Heart Rate
            selectedCount++
        }

        if(workoutProgram.isHRCProgram) {
            root.models[4].setProperty(3, "isSelected", true); // Target Heart Rate
            selectedCount++
        }
    }

    function updateDataDisplay() {
        selectDataModel.clear();
        var c, i;
        var item;

        for (c = 0; c < root.models.length; c++) {
            for(i = 0; i < root.models[c].count; i++) {
                item = root.models[c].get(i);

                if(item.isSelected === true) {
                    selectDataModel.append(item);
                }
            }
        }
    }

    function removeLastEntry() {
        var c, i;
        var item;

        for (c = root.models.length-1; c >=0 ; c--) {
            for(i = root.models[c].count-1; i >=0 ; i--) {
                item = root.models[c].get(i);

                if(item.isSelected === true && c !== 4 ) {  // Don't remove HR related items
                    root.models[c].setProperty(i, "isSelected", false)
                    selectedCount--
                    return
                }
            }
        }
    }

    Component.onCompleted: reset();

    ListModel {
        id: selectDataModel

        ListElement {
            label: QT_TR_NOOP("")
            isSelected: false
            isDisplayed: false
            value: "0"
        }
    }

    // sample ListMode data
    ListModel {
        id: demoModel
        property string heading: qsTr("DISTANCE") + application.emptyString

        ListElement{ label: qsTr("DISTANCE"); isSelected: false; isDisplayed: false; value: "0"}
        ListElement{ label: qsTr("DISTANCE REMAINING"); isSelected: false; isDisplayed: false; value: "0"}
        ListElement{ label: qsTr("DISTANCE CLIMBED"); iisSelected: false; sDisplayed: false; value: "0"}
     }

    Component {
        id: entryDelegate

        Item {
            property int colIndex: ListView.view.colIndex

            height: visible ? ListView.view.height*0.2/*screenHeight * 0.1*/ : 0
            width: ListView.view.width
            visible: model.isDisplayed

            // These are the dimensions of the content that you can actually see
            property real visibleWidth: width*0.9
            property real visibleHeight: height*0.8

            Rectangle {
                width: parent.visibleWidth
                height: parent.visibleHeight
                radius: main.defaultRadius
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                color: isSelected ? colorSelected : "transparent"
                opacity: 1
            }

            NumberLabel {
                id: value
                height: parent.visibleHeight/2
                width: parent.visibleWidth
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                anchors.verticalCenterOffset: -height / 2
                visible: true
                font.pixelSize: height * 0.8
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                text: model.value
            }

            TextLabel {
                id: label
                height: parent.visibleHeight/2
                width: 0.95*parent.visibleWidth
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                anchors.verticalCenterOffset: height / 2
                font.pixelSize: height * 0.4
                horizontalAlignment: root.hAlign
                verticalAlignment: Text.AlignVCenter
                text: {
                    if (model.label === "DISTANCE")
                        return distanceLabelTranslated
                    else if (model.label === "PACE")
                        return paceLabelTranslated
                    else if (model.label === "AVG SPEED")
                        return ((main.isTreadmill || main.isStepmill) ? qsTranslate(root.sourceFile, "AVG SPEED") + "(" + unitsLabelTranslated + ")" : qsTranslate(root.sourceFile, "AVG PACE (RPM)") ) + application.emptyString
                    else if (model.label === "MAX SPEED")
                        return ((main.isTreadmill || main.isStepmill) ? qsTranslate(root.sourceFile, "MAX SPEED") + "(" + unitsLabelTranslated + ")" : qsTranslate(root.sourceFile, "MAX PACE (RPM)") ) + application.emptyString
                    else
                        return qsTranslate(root.sourceFile, model.label) + application.emptyString
                }
                wrapMode: Text.WordWrap
                isHairline: true
            }

            MouseArea {
                id: mouseArea
                enabled: root.selectable
                width: parent.visibleWidth
                height: parent.visibleHeight
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.verticalCenter: parent.verticalCenter
                onClicked: {
                    toggleSelection(parent.colIndex, model.index, model.isSelected)
                    root.wasHRAutoSelected = false
                    buzzer.buttonPress()
                }

                Rectangle {
                    anchors.fill: parent
                    color: main.colorGenericStimulusArea
                    opacity: 0.3
                    visible: main.showStimulusArea
                }
            }
        }

    }

    Component {
        id: colview

        Item {
            id: columnItem

            property variant model

            width: root.colWidth
            height: root.height

            // column header text
            TextLabel {
                id: headingText
                text: columnItem.model ? qsTranslate(root.sourceFile, columnItem.model.heading) : ""
                font.pixelSize: height*0.4
                width: parent.width
                height: root.headerHeight
                horizontalAlignment: root.hAlign
                verticalAlignment: Text.AlignVCenter
                anchors{ top: parent.top; topMargin: 0; left: parent.left ; leftMargin: 0;}
            }

            ScalableImage {
                id: verticalRule
                width: root.separatorWidth
                height: 0.8 * (parent.height - root.headerHeight)
                anchors.top: headingText.bottom
                anchors.topMargin: 5 * main.vScaleFactor
                anchors.right: parent.right
                anchors.rightMargin: -width/2

                fillMode: Image.TileVertically
                horizontalAlignment: Image.AlignHCenter
                verticalAlignment: Image.AlignTop

                sourceName: ruleSourceName
                visible: index < root.models.length - 1
            }

            Rectangle {
                id: horizontalRule
                width: parent.width
                height: 2 * main.vScaleFactor
                color: main.colorUIBorder
                anchors { top: headingText.bottom ; left: parent.left }
            }

            ListView {
                id: colListView
                property int colIndex: index
                interactive: selectable
                clip: true

                width: parent.width
                height: parent.height - root.headerHeight
                delegate: entryDelegate
                anchors{ top: headingText.bottom; topMargin: 0; left: parent.left ; leftMargin: 0 }
                model: columnItem.model
            }
        }
    }

    // row of table columns
    Row {
        spacing: 0
        Repeater {
            model: root.models.length
            delegate: colview

            onItemAdded: { // assign a model from the array to each column
                item.model = root.models[index]
            }
        }
    }
}

