import QtQuick 2.4
import Process 0.1
import VLCVideo 1.0
import FileIO 1.0
import "Television/"
import Components 1.0

// This element represents the television tuner and its channel scan results.
Item {
    id: root
    visible: false

    property bool on: false
    property string mrl
    property real aspect: videoPlayer.aspect > 0 ? videoPlayer.aspect : 1.333
    property bool playing: videoPlayer.playing
    property bool recording: tvRecordingManager.recording

    property alias player: videoPlayer
    property alias fullscreen: videoPlayer.fullscreen
    property bool tuning: videoPlayer.tuning
    property bool muted: false
    property alias channelList: chanlist
    property int currentChannel: -1
    property string currentChannelID: ""
    property string lastChannelID: ""
    property string tunerType
    property string scriptsdir: "scripts"
    property string channelFile: "/data/tv_conf.xml"
    property int numCaptionTracks: videoPlayer.ccTrackCount
    property alias captionTrack: videoPlayer.captionTrack
    property int lastCaptionTrack: 0
    property bool tuningTimedOut: false
    property bool usbResetFailed: videoPlayer.usbResetFailed
    property bool usbReseting: videoPlayer.usbResetInProgress
    property int numUSBErrors: videoPlayer.numUSBErrors
    property bool vlcErrorOccurred: videoPlayer.vlcErrorOccurred
    property bool isIpodInput: false
    property bool bcasCardPresent: false
    property bool proIdiomPresent: false
    property bool verbose: false
    property string tvAlsaDevice: tunerType === "DVB-T2" && fileio.resolveSymlink("/dev/video2") === "/dev/video1" ? "hw:2,0" : "hw:1,0"
    property alias tuneTimes: tuneTimes

    signal clicked()
    signal tuningTimeout()

    function restoreChannel() {
        console.log("Television: Restoring last channel: ", lastChannelID);

        if (lastChannelID !== "") {
            tuneChannelID(lastChannelID);
        }
    }

    function tuneChannelID(id) {

        if(root.usbReseting)
            return

        if (root.currentChannelID !== "") {
            lastChannelID = currentChannelID;
            lastCaptionTrack = captionTrack
        }

        if(id === "IPODINPUT") {
            //special case for IPODINPUT - an AUX channel not in the channel list
            root.isIpodInput = true;
            root.currentChannel = -1;
            root.currentChannelID = "";
            root.mrl = chanlist.getAuxMrl("NTSC_M");

            if (!bluetoothAdapter.connectedHeadset) {
                root.mrl += ":live-caching=15";
            }

            return;
        } else {
            root.isIpodInput = false;
            audioManager.tvRequest();
        }

        var idx = chanlist.findChannel(id);

        if(idx >= 0) {
            root.currentChannel = idx;
            root.currentChannelID = id;
            root.on = true;
        } else {
            root.currentChannelID = "";
        }
    }

    function channelUp() {
        if(root.usbReseting)
            return

        var idx = root.currentChannel;

        if(idx < chanlist.channels.count-1) {
            idx++;
        } else {
            idx = 0;
        }

        var chanid = chanlist.channels.get(idx).id;

        if(chanid) {
            tuneChannelID(chanid);
        }
    }

    function channelDown() {
        if(root.usbReseting)
            return

        var idx = root.currentChannel;
        if(idx > 0) {
            idx--;
        } else {
            idx = channelList.channels.count - 1;
        }

        var chanid = chanlist.channels.get(idx).id;

        if(chanid) {
            root.on = true;
            tuneChannelID(chanid);
        }
    }

    // switch off the TV
    function stop() {
        root.on = false;
    }

    Component.onCompleted: {
        tunerDetect.start();
        bcasDetect.start();
        proIdiomDetect.start();
        chanlist.readChannels(root.channelFile);
    }
    onOnChanged: {
        print("TV " + (on ? "on" : "off"));

        if(on) {
            root.mrl = getTVMrl();
            videoPlayer.play();
        } else {
            tuningTimer.stop();
            videoPlayer.stop();

            // Not performing root hub reset, so set to failed
            if(tv.usbReseting) {
                tv.usbReseting = false
                tv.usbResetFailed = true
            }
        }
    }

    function getChannelObject() {
        return root.channelList.channels.get(currentChannel);
    }

    // get the current channel mrl, given the selected channel number
    function getTVMrl() {
        var mrl;

        if(root.channelList.channels.get(currentChannel)) {
            mrl = chanlist.mrl(currentChannel);
            return mrl;
        } else {
            return "";
        }
    }

    function tunedToCab() {
        return currentChannel >= 0 && channelList.channels.count === 1;
    }

    function getTVChannelName() {
        if(root.channelList.channels.get(currentChannel)) {
            return root.channelList.channels.get(currentChannel).name;
        } else {
            return "";
        }
    }

    function getTVChannelNumber() {
        var item = root.channelList.channels.get(currentChannel);

        if(item) {
            return item.number;
        } else {
            return "";
        }
    }

    onCurrentChannelChanged: {
        if(!on) {
            return
        }

        root.mrl = getTVMrl()
    }
    onMrlChanged: {
        print("TV MRL:" + mrl)
        videoPlayer.mrl = ""
        videoPlayer.mrl = mrl
    }

    FileIO {
        id: fileio
    }

    ChannelList {
        id: chanlist
        channelFile: root.channelFile
    }

    onTuningChanged: {
        if(root.tuning) {
            if (tv.on) {
                tuningTimer.restart()
                root.tuningTimedOut = false
            }
        } else {
            tuningTimer.stop()
        }
    }

    onNumCaptionTracksChanged: {
        if (tv.on && tv.numCaptionTracks > 0) {
            if(lastCaptionTrack < tv.numCaptionTracks)
                captionTrack = lastCaptionTrack
            else {
                captionTrack = 0
                lastCaptionTrack = captionTrack
            }
        }
    }

    Timer {
        id: tuningTimer
        interval: 7000
        onTriggered: {
            root.tuningTimedOut = true
            //root.tuningTimeout()
            //videoPlayer.stop()
            //videoPlayer.play()
	    print("Tuning Timeout Currently Disabled")
        }
    }

    ListModel {
        id: tuneTimes
        property int numberTimesToStore: 10
        function appendTuneTime(timeMs) {
            if (tuneTimes.count >= numberTimesToStore) {
                tuneTimes.remove(0)
            }
            tuneTimes.append({"time": timeMs})
        }
    }

    VLCVideo {
        id: videoPlayer
        objectName: "television"

        property bool fullscreen  // set to true to expand into fullscreenFrame

        commandLineArgs: {
            var args = ["--video-filter=deinterlace", "--deinterlace=-1", "--v4l2-use-libv4l2" ]
            args = args.concat(["--avcodec-skiploopfilter=4"])
            //args = args.concat(["--dropped-frame-stats", "-vvv", "--ts-silent"])
            if (root.verbose) {
              args = args.concat(["-vvvv"])
            } else {
              //args = args.concat([ "--quiet", "--ts-silent" ])
            }

            if(options.tvHwAccel) {
                args = args.concat(["--avcodec-hw=vaapi"])
            } else {
                args = args.concat(["--avcodec-hw=none"])
            }

            if(options.tvNoFrameDrop) {
                args = args.concat(["--no-drop-late-frames", "--avcodec-skip-frame=1"])
            }

            return args
        }
        onMediaError: root.tuningTimedOut = true
        debug: true
        // don't stream audio from tuner if it is already coming from RCA jacks or iPod
        audioDevice: root.isIpodInput && !bluetoothAdapter.connectedHeadset ? "hw:0,0" : root.tvAlsaDevice
        deinterlace: false
        muted: root.muted
        onPlaybackStarted: {
            if(options.autoChannelChangeInterval > 99) {
                tuneTimePopup.message = tuneTime/1000.0 + " seconds"
                tuneTimePopup.showWithTimeout(1)
            }
            console.log("Tune time:", tuneTime)
            tuneTimes.appendTuneTime(tuneTime)
            console.log(tuneTimes.count)
            var chan = root.getChannelObject()
            var freq = chan.freq
            console.log("TUNETIME", freq, tuneTime, tv.mrl);
        }
    }

    MessagePopup {
        id: tuneTimePopup
    }

    Process { id: tunerDetect
        program: root.scriptsdir + "/tunertype.sh"
        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
            output = output.trim()
            var tuners = output.split("\n")

            for (var i = 0; i < tuners.length; i++) {
                root.tunerType = tuners[i]
            }

            print("tunerDetect process: Tuner type: " + output)
        }
    }

    Process {
        id: bcasDetect
        program: "scripts/detect_bcas.sh"
        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
            output = output.trim()
            console.log( "bcasDetect process (STDOUT): BCAS card present = " + output)
            root.bcasCardPresent = (output === "yes")
        }
    }

    Process {
        id: proIdiomDetect
        program: "scripts/detect_proIdiom.sh"
        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
            output = output.trim()
            console.log( "proIdiomDetect process (STDOUT): proIdiom detected = " + output)
            root.proIdiomPresent = (output === "yes")
        }
    }

    /* Remove root hub reset for now and just reboot the console
    function startRootHubReset () {
        if(root.usbReseting) {
            tvTestHud.usbhubcount++
            console.log("**********************************************************************************Calling resetUsbRootHub.start() with arguments" + resetUsbRootHub.arguments)
            resetUsbRootHub.start()

            // Not performing root hub reset, so set to failed
            tv.usbReseting = false
            tv.usbResetFailed = true
        }
    }

    Process { id: resetUsbRootHub
        program: "scripts/usb-reset.sh"

        onStarted: {
            console.log("USB root hub reset started")
        }

        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
            output = output.trim()
            console.log( "resetUsbRootHub process (STDOUT):" + output)

            // Not performing root hub reset, don't reset local hub either
//            console.log("Now reset local hub")
//            videoPlayer.resetHub(true);
        }

        onReadyReadStandardError: {
            var output = String(readAllStandardError())
            output = output.trim()
            console.log( "resetUsbRootHub process (STDERR):" + output)
        }
    }
*/

    function usbError() {

        triggerUSBError.start()
    }

    Process { id:triggerUSBError
        program: "scripts/trigger-reset.sh"
        arguments: "0"

        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
            output = output.trim()
            console.log( "triggerUSBError process (STDOUT):" + output)
        }

        onReadyReadStandardError: {
            var output = String(readAllStandardError())
            output = output.trim()
            console.log( "triggerUSBError process (STDERR):" + output)
        }
    }

}
