import QtQuick 2.4
import VLCVideo 1.0
import WorkoutRunData 1.0
import "string_format.js" as Format
import Components 1.0

// Video window to play video
Rectangle {
    id: root
    color: "white"
    anchors.fill: parent

    property ListModel c360Modes: main.isLateralTrainer ? c360Modes_LateralTrainer : c360Modes_Elliptical
    property alias videoMRL: videoPlayer.mrl
    property variant c360program: workoutProgram.c360Program
    property int phase: c360program.currentPhase
    property string videoPath: "/data/video/Cardio360/" + (main.isLateralTrainer ? "Traverse" : "Spectrum")
    property string videoSubpath: options.modestyMode ? "Male" : (Math.random() >= 0.5 ? "Female" : "Male")
    property string introVideo: "Intro Clip.mp4"
    property int segmentTime: c360program.phaseTimeRemaining
    property bool skipIntroMode: main.isLateralTrainer
    property bool inIntroMode: !skipIntroMode

    signal phaseTitleChanged(string text)
    signal playbackComplete()

    property bool isMenuDisplayed: false
    signal action()

    function beginWorkout() {
        root.inIntroMode = false;

        if (setVideoTimer.running) {
            setVideoTimer.stop();
        }

        workoutManager.start();
        root.phaseChanged(phase);
    }

    Connections {
        target: workoutManager
        onWorkoutModeChanged: {
            if (mode === WorkoutRunData.COOL_DOWN_MODE) {
                if (root.inIntroMode) {
                    workoutManager.resume()
                }

                root.inIntroMode = false
            }
        }
    }

    Component.onCompleted: setVideoTimer.start() // Set video on delay to avoid race condition where video is not set

    Timer {
        id: setVideoTimer
        interval: 300
        running: false
        onTriggered: {
            if (skipIntroMode) {
                beginWorkout()
            } else {
                videoMRL = videoFullPath(introVideo)
                bannerForIntro.display()
            }
        }
    }

    onPhaseChanged: {
        if (root.inIntroMode) {
            return
        }

        var text = qsTranslate("C360ViewContainer", c360Modes.title(phase))
        root.phaseTitleChanged(text)
        videoMRL = videoFullPath(c360Modes.get(phase).video)
    }

    function videoFullPath(path) {
        return "file://" + root.videoPath + "/" + videoSubpath + "/" + path;
    }

    function stopVideo() {
        videoPlayer.stop();
    }

    SimpleMessageBanner {
        id: bannerForIntro
        beep: false
        x:185
        y:170
        width:1000
        dismissable: false
        z: 10000

        property int index: 0

        ListModel {
            id: messageList
            ListElement { text: QT_TR_NOOP("Welcome to Cardio360") }
            ListElement { text: QT_TR_NOOP("Your Total Body Workout") }
        }

        function display() {
            buzzer.buttonPress();
            index = 0;
            showMessage(qsTr(messageList.get(index).text), 3);
        }

        onTimedOut: {
            index++

            if (index >= messageList.count) {
                index = 0
                return
            }

            showMessage(qsTr(messageList.get(index).text), 3)
        }
    }

    VLCSurface {
        id: videoSurface
        backgroundColor: "white"
        player: videoPlayer
        aspect: videoPlayer.aspect
        playing: !videoPlayer.fullscreen && videoPlayer.playing
        anchors.fill: parent
    }

    VLCVideo {
        id: videoPlayer

        property bool fullscreen

        debug: false
        deinterlace: false
        loop: !inIntroMode
        onPlayCompleted: {
            if (inIntroMode) {
                beginWorkout()
            }
        }
    }

    ScalableImage {
        id: motionMan
        visible: !inIntroMode
        anchors {
            left: parent.left
            leftMargin: 40 * main.hScaleFactor
            top: parent.top
            topMargin: 85 * main.vScaleFactor
        }
        fillMode: Image.PreserveAspectFit
        smooth: true
        height: 260 * main.vScaleFactor
        width: 262 * main.hScaleFactor
        sourceName: "c360/MotionMan/" + c360Modes.get(phase).icon.replace(/\.[^/.]+$/, "");
    }

    Text {
        id: instText
        width: 300 * main.hScaleFactor
        visible: !inIntroMode
        anchors {
            right: parent.right
            rightMargin: 55 * main.hScaleFactor
            bottom: segRect.top
            bottomMargin: 20 * main.vScaleFactor
        }
        wrapMode: Text.WordWrap
        text: qsTranslate("C360ViewContainer", c360Modes.instructions(phase)) + application.emptyString
    }

    Text {
        id: guideTextForIntro
        wrapMode: Text.WordWrap
        width: 230 * main.hScaleFactor
        visible: inIntroMode
        anchors {
            top: parent.top
            topMargin: 175 * main.vScaleFactor
            right: parent.right
            rightMargin: 100 * main.hScaleFactor
        }
        text: qsTr("Watch the on-screen video for instructions to get the most out of your workout.") + application.emptyString
    }

    Item {
        id: segRect
        visible: !inIntroMode
        width: 270 * main.hScaleFactor
        height: childrenRect.height
        anchors {
            top: parent.top
            topMargin: 275 * main.vScaleFactor
            right: parent.right
            rightMargin: 96 * main.hScaleFactor
        }

        Text {
            id: segTimeLabel
            width: 270 * main.hScaleFactor
            horizontalAlignment: Text.AlignHCenter
            anchors.horizontalCenter: parent.horizontalCenter
            text: qsTranslate("C360View", "SEGMENT TIME REMAINING") + application.emptyString
            font.pointSize: 14 * main.vScaleFactor
            wrapMode: Text.Wrap
        }

        Text {
            id: segTimeIndicator
            anchors {
                top: segTimeLabel.bottom
                horizontalCenter: parent.horizontalCenter
            }
            text: Format.formatTimeString(root.segmentTime)
            font.pointSize: 45 * main.vScaleFactor
            font.bold: false
        }
    }

    Item {
        id: skipIntroText
        visible: root.inIntroMode
        width: childrenRect.width
        height: childrenRect.height
        anchors {
            right: parent.right
            rightMargin: 105 * main.hScaleFactor
            bottom: parent.bottom
            bottomMargin: 35 * main.vScaleFactor
        }

        Text {
            id: skipText1
            text: qsTr("SKIP INTRO") + application.emptyString
            font.pixelSize: 20 * main.vScaleFactor
        }

        Text {
            text: "X"
            anchors.left: skipText1.right
            anchors.leftMargin: 5 * main.hScaleFactor
            font.pixelSize: 24 * main.vScaleFactor
        }

        MouseArea {
            anchors.fill: parent
            onClicked: {
                bannerForIntro.hide()
                beginWorkout()
                buzzer.buttonPress()
                root.action()
            }

            Rectangle {
                anchors.fill: parent
                color: main.colorGenericStimulusArea
                opacity: 0.3
                visible: main.showStimulusArea
            }
        }
    }
}
