import QtQuick 2.7
import TouchAppEnumerations 1.0
import WorkoutRunData 1.0
import WorkoutManager 1.0
import QtQuick.Layouts 1.1
import Components 1.0
import "string_format.js" as StrFormat

StandardPage {
    id: root
    width: main.width
    height: main.height
    title: root.workoutName + qsTr(" SUMMARY") + application.emptyString
    
    image_width: 225/1366
    showBackButton: false

    property string workoutName: qsTr("WORKOUT") + application.emptyString
    property string workoutElapsed: "00:00"

    property bool isUsbDeviceLoaded: storageMgr.usbDeviceLoaded
    property int timeLeft: 0
    property bool saveHasHappened: false
    property int advertiseTimeOout: 60
    property bool dialogVisible: false

    signal close()
    signal save()
    signal advertise()
    signal connected()
    signal disconnect()

    Connections {
        target: root
        onConnected: bleDialog.show()
        onAdvertise: {
            bleDialog.show()
            ble.startAdvertiseing()
        }
        onDisconnect: {
            ble.stopAdvertising()
            bleDialog.hide()
        }
    }

    onClose: disconnect()

    function doSave() {
        if(isUsbDeviceLoaded) {
            save();
        } else {
            advertise();
            ble.advertising = true;
        }
    }

    function convertUnits(speedMiles) {
        return workoutScreen.unitState === "METRIC" ? Globals.mileToKm(speedMiles) : speedMiles;
    }

    function convertMeters(feet) {
        return workoutScreen.unitState === "METRIC" ? Globals.feetToMeters(feet).toFixed(0) : feet;
    }

    function setData(workoutData) {
        dataSummaryTable.reset();
        dataSummaryTable.updateDataValues();
        fitTestDataTable.setData(workoutData);

        if(fitTestDataTable.visible)
            root.workoutName = fitTestDataTable.workoutName;

        // prepare data for bluetooth advertisement
        root.workoutElapsed = workoutData.totalTime;
    }

    Item {
        id: contentArea
        width: 1100 * main.hScaleFactor
        height: root.height
        anchors {
            horizontalCenter: parent.horizontalCenter
            horizontalCenterOffset: 43 * main.hScaleFactor
            top: parent.top
        }

        Item {
            id: tableBackground

            anchors.left: toolbarRow.left
            anchors.top: parent.top
            anchors.topMargin: 120 * main.vScaleFactor
            anchors.right: homeScreenButton.right
            anchors.bottom: toolbarRow.top
            anchors.bottomMargin: 50 * main.vScaleFactor

            DataSummaryTable {
                id: dataSummaryTable
                selectable: false
                tableWidth: width - 2*tableHMargin
                anchors.fill: parent
                visible: !fitTestDataTable.visible
            }

            FitTestSummaryTable {
                id: fitTestDataTable
                anchors.fill: parent

            }
        }

        Row {
            id: toolbarRow
            anchors.bottom: contentArea.bottom
            anchors.bottomMargin: 20 * main.vScaleFactor
            anchors.left: contentArea.left
//            anchors.leftMargin: 287 * main.hScaleFactor

            height: 93 * main.vScaleFactor
            spacing: 30 * main.hScaleFactor

            Rectangle {
                id: resetTimerButton
                visible: true
                width: 140 * main.hScaleFactor
                height: parent.height
                border.color: main.colorUIBorder
                border.width: main.widthBorder
                color: main.colorButtonBackground
                radius: main.defaultRadius

                NumberLabel {
                    id: countDownText
                    anchors {
                        top: parent.top
                        topMargin: 10 * main.vScaleFactor
                        horizontalCenter: parent.horizontalCenter
                    }
                    text: StrFormat.formatTimeString(root.timeLeft)
                    font.pixelSize: 36 * main.vScaleFactor
                }

                TextLabel {
                    id: resetTimerText
                    anchors {
                        bottom: parent.bottom
                        bottomMargin: 13 * main.vScaleFactor
                        horizontalCenter: parent.horizontalCenter
                    }
                    width: 0.9*parent.width
                    text: qsTr("RESET TIMER") + application.emptyString
                    font.pixelSize: 13 * main.vScaleFactor
                    wrapMode: Text.WordWrap
                    horizontalAlignment: Text.AlignHCenter
                }
            }

            StandardButton {
                id: exportDataButton
                width: 100 * main.hScaleFactor
                height: parent.height
                visible: false // Hide functionality until 3rd party apps (eg. Wahoo) are developed to export data to
                buttonText: qsTr("Export") + application.emptyString
                onClicked: root.doSave()

                icon.sourceName: "Bluetooth/BluetoothIcon";
            }

            StandardButton {
                id: signInRegisterButton
                height: parent.height
                enabled: main.loginAvailable
                visible: main.loginAvailable
                buttonText: qsTr("SIGN IN / REGISTER") + application.emptyString
                onClicked: main.state = "LoginState"

                icon.sourceName: "HomeScreen/SignInRegisterIcon"
            }

            Column {
                id: signInText
                height: parent.height
                width: 250 * main.hScaleFactor
                spacing: 6 * main.vScaleFactor

                enabled: main.loginAvailable
                visible: main.loginAvailable

                TextLabel {
                    id: signInRegisterText
                    text: qsTr("SPEED UP YOUR WORKOUT") + application.emptyString
                    font.pixelSize: 18 * main.vScaleFactor
                }

                Row {
                    id: bulletRow1
                    height: implicitHeight
                    width: implicitWidth

                    Text {
                        id: bullet1
                        anchors.verticalCenter: signInRegisterTextBullet1.verticalCenter
                        text: "●"
                        color: main.colorFontLight
                        font.pixelSize: 4 * main.vScaleFactor
                    }

                    Text {
                        id: signInRegisterTextBullet1
                        anchors.top: parent.top
                        text: qsTr(" Automatic export to your app!")
                        color: main.colorFontLight
                        font.pixelSize: 12 * main.vScaleFactor
                    }
                }

                Row {
                    id: bulletRow2
                    height: implicitHeight
                    width: implicitWidth

                    Text {
                        id: bullet2
                        anchors.verticalCenter: signInRegisterTextBullet2.verticalCenter
                        text: "●"
                        color: main.colorFontLight
                        font.pixelSize: 4 * main.vScaleFactor
                    }

                    Text {
                        id: signInRegisterTextBullet2
                        anchors.top: parent.top
                        text: qsTr(" More workout statistics!")
                        color: main.colorFontLight
                        font.pixelSize: 12 * main.vScaleFactor
                    }
                }

                Row {
                    id: bulletRow3
                    height: implicitHeight
                    width: implicitWidth

                    Text {
                        id: bullet3
                        anchors.verticalCenter: signInRegisterTextBullet3.verticalCenter
                        text: "●"
                        color: main.colorFontLight
                        font.pixelSize: 4 * main.vScaleFactor
                    }

                    Text {
                        id: signInRegisterTextBullet3
                        anchors.top: parent.top
                        text: qsTr(" Sign up today!")
                        color: main.colorFontLight
                        font.pixelSize: 12 * main.vScaleFactor
                    }
                }
            }
        }

        StandardButton {
            id: homeScreenButton
            anchors.right: contentArea.right
            anchors.rightMargin: 70 * main.vScaleFactor
            anchors.top: toolbarRow.top
            width: 120 * main.hScaleFactor
            height: toolbarRow.height
            visible: true
            buttonText: qsTr("HOME") + application.emptyString
            onClicked: close()

            icon.sourceName: "Components/HomeIcon"
        }
    }

    BlueToothLowEnergy {
        id: ble
        Component.onCompleted: console.log("ble." +ble.dummy)
    }

    BluetoothTransmitPopup {
        id: bleDialog
    }
}
