/** @file main.qml
    @brief Manage main display and transitions to workout menus and run screens.
*/

import QtQuick 2.7
import QtQuick.Window 2.2 // For Screen
import WorkoutManager 1.0
import StorageMgr 1.0
import StatisticManager 1.0
import ErrorManager 1.0
import WorkoutRunData 1.0
import Process 0.1
import AMixer 1.0
import DataExport 1.0
import Bluetooth 1.0
import HeartRateManager 1.0
import RecoveryManager 1.0
import SapManager 1.0
import Components 1.0
import FileIO 1.0
import "LanguageMenu"
import "Diagnostics"
import "MediaBrowser"
import "string_format.js" as StrFormat
import "Television"
import "Keypad"
import "trueui"

import VLCVideo 1.0

/**
 * @brief the base QML file for the application.
 */

Item {
    id: main

    /* Some global properties */
    property string colorUIBorder: "#BA3E3E"
    property string colorSelected: "#401515"
    property string colorDialogBackground: "black"
    property string colorToolbarBackground: "#383230"
    property string colorButtonBackground: "transparent"
    property string colorGenericStimulusArea: "red"
    property string colorKnobStimulusArea: "blue"
    property string colorTrackMouseStimulusArea: "green"
    property string colorHighlightBackground: "#401515"
    property string colorFontGreen: "#5FFF00"
    property string colorFontLight: "white"
    property string colorFontDark: "black"
    property string colorDiagnosticEdges: "black"

    property real opacityDialog: 0.9
    property int widthDialog: 800 * main.hScaleFactor
    property int heightDialog: 350 * main.hScaleFactor
    /* To make our lives easier, make border width an even number, or else child
     * elements will clip borders due to borders being rendered at half pixels
     */
    property real widthBorder: ((width * 0.001) < 2 ? 2 : width * 0.001)
    property real defaultRadius: 10 * main.scaleFactor
    
    /**
     * Use SVG format images in ScalableImages
     */
    property bool useSVG: false

    /**
     * Set width and height based on screen size
     */
    width: Screen.width
    height: Screen.height
    /**
     * Default to 1366x768 with a pixel density of 2.91251 px/mm
     */
    readonly property int defaultWidth: 1366
    readonly property int defaultHeight: 768
//    readonly property real defaultPixelDensity: 2.91251
    /**
     * Calculate scale factors based on ratio of current screen to default and density of pixels
     */
    property real hScaleFactor: (width/defaultWidth)//*(Screen.pixelDensity/defaultPixelDensity)
    property real vScaleFactor: (height/defaultHeight)//*(Screen.pixelDensity/defaultPixelDensity)
    property real scaleFactor: Math.min(hScaleFactor, vScaleFactor)

    property bool modestyMode: options.modestyMode

    /**
     * @brief type:bool Boolean that determines if the login button should be visible
     */
//    property bool loginAvailable: true
    property bool loginAvailable: false

    /**
     * @brief type:bool Boolean that determines if a user is logged in.
     */
    property bool loggedIn: false

    /**
     * @brief type:variant Property that contains the topmenu model.
     */
    property variant topmenu

    /**
     * @brief type:variant Property that contains the All Workouts menu.
     */
    property variant allWorkoutsMenu

    /**
     * @brief type:bool Property that determines if a dialog box is shown.
     */
    property bool _dialogShowing: false

    /**
     * @brief type:bool Property that determines if the block keys are shown.
     */
    property bool blockKeys: restartRequired || dialogManager.anyAreBlockingKeys || popupManager.estopShown || main.quickStartLoading

    /**
     * @brief type:bool Property that determines if the stimulus area is shown.
     */
    property bool showStimulusArea: options.showStimulusArea

    /**
     * @brief type:int Property that determines the volume level to use.
     */
    property int volume: 5

    /**
     * @brief type:real Property that determines the time factor.
     */
    property real timeFactor: 1.0

    /**
     * @brief type:string Property that contains the application's serial number.
     */
    property string serialNumber: application.serialNumber

    /**
     * @brief type:string Property that contains the application's product version.
     */
    property string productVersion: application.productVersion

    /**
     * @brief type:bool Property that determines if a treadmill is being used.
     */
    property bool isTreadmill: machineinfo.productTypeString === "Treadmill"

    /**
     * @brief type:bool Property that determines if the machines used is an alpine runner.
     */
    property bool isStepmill: machineinfo.productTypeString === "Stepmill"

    /**
     * @brief type:bool Property that determines if the application's layout is right to left.
     */
    property bool rightToLeft: Qt.application.layoutDirection === Qt.RightToLeft

    /**
     * @brief type:string Property that determines the currently selected channel.
     */
    property string channelSelected: ""

    /**
     * @brief type:bool Property that determines if a restart of the machine is required.
     */
    property bool restartRequired: false  // Show error message and make user power cycle
    property bool rebootRequired: false  // Automatically reboot the console at the end of the workout

    /**
     * @brief type:bool Property that determines if a firmware update is required.
     */
    property bool firmwareUpdateRequired: false

    /**
     * @brief type:bool Property that determines if the language button is visible.
     */
    property bool _hideLanguageButton: false

    /**
     * @brief type:int Property that holds the current screen shot count.
     */
    property int screenShotCount: 0

    /**
     * @brief type:string Property that holds the current console type such as Transcent10.
     */
    property string consoleType: application.consoleType

    /**
     * @brief type:bool Property that determines if each key press will be logged.
     */
    property bool _debugKeys: true

    /**
     * @brief type:bool Property that determines if effective radiated power (ERP) is supported.
     */
    property bool erpIsSupported: true

    /**
     * @brief type:bool Property that determines if loading workout from clicking the Quick Start button.
     */
    property bool quickStartLoading: false

    /**
     * @brief type:bool Property that determines if the machine meets CSA certification?
     */
    property bool isCSA: machineinfo.modelString === "TS1000" || machineinfo.modelString === "XS1000"

    /**
     * @brief type:bool Property that determines if the machine used is a lateral trainer.
     */
    property bool isLateralTrainer: machineinfo.modelString === "TL1000" || machineinfo.modelString === "XL1000"

    /**
     * @brief type:bool Property that determines if the machines used is an alpine runner.
     */
    property bool isAlpineRunner: machineinfo.modelString === "TI1000"

    /**
     * @brief type:bool Property that determines if the media browser is shown.
     */
    property bool showMediaBrowser: false

    /**
     * @brief type:bool Property that determines connection should be terminated
     */
    property bool killOnConnect: false

    /**
     * @brief type:variant Property that holds the home screen.
     */
    property variant home_screen: home_screen_loader.item

    /**
     * @brief type:variant Property that holds the workout menu.
     */
    property variant workout_menu: workout_menu_loader.item

    /**
     * @brief type:string Property that holds the lower screen board type.
     */
    property string lowerSystem: ""

    /**
     * @brief type:bool Property that determines if a user may begin a workout.
     */
    property bool canStartWorkout: true

    /**
      * @brief type:bool Property that determines if the user wants to select a new workout in the middle of a workout
      */
    property bool isNewWorkoutSelected: false

    /**
      * @brief type:bool Property that determines if the console is the primary console in a group of replica consoles
      */
    property bool isPrimary: false

    property int screenHeight: height
    property int screenWidth: width

    focus: state == "HomeState" || state == "LoginState" || state == "RegisterState" || state == "DiagnosticState"

    /**
     * @brief Signal that indicates the state is to be reset.
     */
    signal reset()

    Component.onCompleted: {
        // The translation of the string "QT_LAYOUT_DIRECTION" should be "RTL" or "LTR" in the touch_*.ts files depending on the orientation of the language
        QT_TRANSLATE_NOOP("QApplication", "QT_LAYOUT_DIRECTION");
        // start on the home screen
        print("***** Application Start v" + application.appVersion + " *****");
        state = "HomeState";
        print("Getting Bios Version...");
        getBiosVerProcess.start();
        var lang = options.defaultLanguage;
        application.selectLanguage(lang);
        home_screen.updateLanguageIcon(lang);
        if(qsTranslate("QApplication", "QT_LAYOUT_DIRECTION") === "RTL") {
            main.rightToLeft = true
        } else {
            main.rightToLeft = false
        }

        console.log("language default: " + lang + " with orientation " + (main.rightToLeft ? "RTL" : "LTR"));

        workout_menu.selectWorkoutMenu();
        buzzer.buzzerEnabled = !options.disableBuzzer

        /* On start of application, set simulationMode and ignoreFaults to false */
        if (!options.manufacturingTestModeEnabled) {
            options.simulationMode = false;
            options.ignoreFaults = false;
        } else {
            options.ignoreFaults = true;
        }

        if (application.crashRecovery) {
            logUploadPrompt.show();
            logUploadPrompt.start();
        }

        bluetoothDeviceAudioWatcher.start()

        if(options.triggerWorkoutAutoRestart) {
            // Automatically start Quickstart for TV testing
            console.log("===================================> Quickstart starting in " + autostartTimer.interval + " ms")
            autostartTimer.start()
        }

        // Reset Salt minion in case it got corrupted somehow
        resetMinion.start()
    }

    Process {
        id: resetMinion
        program: "sudo"
        arguments: ["bash", "/srv/scripts/reset_minion.sh"]
        onStarted: {
            console.log("Resetting salt minion")
        }
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim();
            console.log("Reset Salt Minion: ", data);
        }
        onFinished: {
            console.log("Reset salt minion complete")
        }
    }

    Timer{
        id: autostartTimer
        // if roo-hub is being reset, wait 3 minutes else 31 seconds
        interval: tv.usbReseting === true ? 180000 : 31000
        running: false
        repeat: false
        onTriggered:{
            if (main.rebootRequired)
                return

            if(!tv.usbReseting) {
                quickStart()
            } else {
                autostartTimer.restart()
            }
        }
    }

    Timer{
        id: autostopTimer
        interval: 1800000  //30 minutes
        running: false
        repeat: false
        onTriggered:{
            workoutScreen.stopWorkout();
            autostartTimer.start()
        }
    }


    Component.onDestruction: {
        if (bluetoothAdapter.connected)
            bluetoothAdapter.disconnect()
        if (bluetoothAdapter.connectedHeadset)
            bluetoothAdapter.disconnectHeadset()
        console.log("Main: Component destroyed")
    }

    // update the layout direction when the language changes
    Connections {
        target: application
        onLanguageChanged: {
            if(qsTranslate("QApplication", "QT_LAYOUT_DIRECTION") === "RTL") {
                main.rightToLeft = true
            } else {
                main.rightToLeft = false
            }
            console.log("language orientation changed to: " + (main.rightToLeft ? "RTL" : "LTR"));
        }
    }

    Binding { target: application; property: "swipeStartX"; value: 0; when: main.state != "RunState" }
    Binding { target: application; property: "swipeEndX"; value: main.width; when: main.state != "RunState" }
    Binding { target: application; property: "swipeStartY"; value: 0; when: main.state != "RunState" }
    Binding { target: application; property: "swipeEndY"; value: main.height; when: main.state != "RunState" }

    Process {
        id: getBiosVerProcess
        Component.onCompleted: start()
        program: "sudo"
        arguments: ["cat", "/sys/class/dmi/id/bios_version"]
        onReadyReadStandardOutput: {
            var version = String(readAllStandardOutput())
            version = version.trim()
            print("Bios version is " + version)
        }
    }

    NetworkConnections {
        id: netConnections
        onIsRefreshingChanged: {
            if (!isRefreshing) {
                console.log("NetworkConnections: Finished refreshing connections")
                netConnections.printStatus()
            } else {
                console.log("NetworkConnections: Started refreshing connections")
            }
        }
    }

    Connections {
        target: nucleus
        onNucleusConnectionChanged: {
            if(connection == true) {
                nucleus.startInitializationProcess()
            }
        }
        onInitializationProcessFinished: {
            if(result == true) {
                workout_menu.selectWorkoutMenu()
                console.log("Nucleus: Diagnostics = " + options.loggingEnabled)
                nucleus.setDiagnostics(options.loggingEnabled)
                nucleus.cabSetEnable(options.cabEnabled)

                // Retrieve Nucleus error logs
                console.log("Retrieving Nucleus fault log")
                errorManager.retrieveFaultLogs()
            }
        }

        onSystemReadyChanged: {
            console.log("Nucleus: System Ready = " + nucleus.systemReady)
            if(nucleus.systemReady === true) {
                canStartWorkout = true                
            } else {
                canStartWorkout = false;
            }
        }
    }

    Connections{
        target: tv
        onTunerTypeChanged: {
            switch(tv.tunerType){
            case "ATSC":
                erpIsSupported = true;
                break;
            default:
                erpIsSupported = true;
                break;
            }
        }
    }

    /**
     * @brief Prints debug keys to the console log.
     * @param type:string keyname The name of the key pressed.
     * @param type:string objectname The name of the object.
     * @param type:string notes Any addtional information on the key.
     */
    function debugKeys(keyname, objectname, notes) {
        if (main._debugKeys) {
            console.log("KEYPRESS: ",  keyname, objectname, notes);
        }
    }

    /**
     * @brief Puts the ERP to sleep.
     */
    function erpSleep() {
        if (lowerSystem === "NUCLEUS") {
            nucleus.erpSleep()
        }
    }

    DialogManager {
        id: dialogManager
    }

    Process { id: syncDisk
        program: "/bin/sync"
        block: true
        blockTimeout: 1000 * 60 * 60; // 1 hour
        onFinished: {
            syncNoBlockTimer.restart()
        }
    }

    Timer {
        id: syncNoBlockTimer
        interval: 1000
        repeat: false
        running: false
        onTriggered: {
            syncNoBlock.start()
        }
    }

    Process {
        id: syncNoBlock
        program: "/bin/sync"
    }

    MessagePopup {
        id: beltStoppingPopup
        blockKeys: false
        isTransparent: true

        /**
         * @brief type:bool Property that determines if the pop-up window is to be displayed.
         */
        property bool showPopup: main.state === "HomeState" &&
                                 ((isTreadmill && popupManager.estopShown === false && (machinecontrol.isBeltMoving === true || machinecontrol.isInclineMoving === true || (lowerSystem === "NUCLEUS" && nucleus.systemReady === false)))
                                   || (isStepmill && popupManager.estopShown === false && (machinecontrol.isBeltMoving === true || (lowerSystem === "NUCLEUS" && nucleus.systemReady === false))))

        onShowPopupChanged: {
            if (showPopup) {
                show();
            }
            else {
                hide();
            }
        }
        message: qsTr("Please wait...") + application.emptyString
        Component.onCompleted: hide()
    }

    MessagePopup {
        id: blockUseMessage

        /**
         * @brief type:bool Property that determines if the pop-up window is to be displayed.
         */
        property bool showPopup: main.state !== "DiagnosticState" && main.state !== "SetupWizardState" && (restartRequired || home_screen.buttonsEnabled === false)

        onShowPopupChanged: {
            if (showPopup) {
                show();
            }
            else {
                hide();
            }
        }
        message: {
            if (restartRequired) {
                return (application.crashRecovery ? qsTr("Power Cycle Required") : tv.usbResetFailed ? qsTr("TV Error Detected.") + "\n" + qsTr("Power Cycle Required.") : qsTr("Power Cycle Required to Complete Update")) + application.emptyString;
            } else if (!machineinfo.isReady) {
                return qsTr("Starting up...") + application.emptyString;
            } else if (!home_screen.buttonsEnabled) {
                if ((machineinfo.productTypeString === "") && (options.ignoreFaults == false)) {
                    return qsTr("Console Configuration Required.") + application.emptyString;
                } else if ((machineinfo.productTypeString === "Treadmill") && (machinecontrol.isCalibrated === false) && (options.ignoreFaults === false)) {
                    return qsTr("Treadmill Calibration Required.") + application.emptyString;
                } else if ((machineinfo.productTypeString === "") && (options.ignoreFaults == true)) {
                    return qsTr("Product Configuration Required.") + application.emptyString;
                } else {
                    return qsTr("Complete Configuration to Enable Buttons") + application.emptyString;
                }
            } else {
                return qsTr("Complete Configuration to Enable Buttons") + application.emptyString;
            }
        }
        Component.onCompleted: if (showPopup) show(); else hide()
        isTransparent: true
    }

    MiscStats {
        id: miscStats
    }

    WorkoutStats {
        id: workoutStats
    }

    UserData { id: userData
        loggedIn: false
    }

    DataExport {
        id: dataExport
        workoutProgram: workoutProgram
        workoutData: aWorkoutRunData
        userData: userData
        profile1: workoutProgram.profile1
        profile2: workoutProgram.profile2
        productModel: machineinfo.modelString
//        machineID: netpulse.machineID
        serialNumber: main.serialNumber
        runMiles: statisticManager.totalDistance
        runHours: statisticManager.runHours / 3600  // stats manager keeps "runHours" in seconds
        modelID: 0 //configurationData.csafeID
        version: application.appVersion
        Component.onCompleted: print("ip address:" + ipAddress)
    }

    SignIn {
        id: signin_window
        z:99
        visible: (main.state === "LoginState");
    }

    CreateProfile {
        id: createprofile_window
        visible: (main.state === "RegisterState");
    }

    Connections{
        target: tv
        onTunerTypeChanged: {
            switch(tv.tunerType){
                case "ATSC":
                    erpIsSupported = true;
                    break;
                default:
                    erpIsSupported = true;
                    break;
            }
        }
    }

    // expire Login after 30 seconds with no activity on homescreen
    Timer {
        id: loginTimeoutTimer
        running: (main.state == "HomeState")
        interval: 30 * 1000
        onTriggered: { // TODO: write a logout function
            reset();
        }
        /**
         * @brief Calls the restart function if currently running.
         */
        function reset() {
            if(running) {
                restart();
            }
        }
    }

    Connections {
        // reset the login timer when there's a screen touch
        target: application
        onScreenTouched: loginTimeoutTimer.reset()
    }

    AMixer {
        id: amixer
    }

    StatisticManager {
        id:statisticManager
        onTotalDistanceChanged: errorManager.updateMileage(totalDistance)
        Component.onCompleted: {
            setNucleusObject(nucleus);
        }
    }

    MessagePopup {
        id: infoPopup
        dismissable: true
        dismissAnywhere: true
        isTransparent: true
    }

    MessageBanner {
        id: infoBanner
        z: 10000
        anchors.horizontalCenter: main.horizontalCenter
        y:0
        dismissable: true
        dismissAnywhere: true
        isTransparent: true
    }

    SimpleMessageBanner {
        id: viewContainerBanner
        z: 10000
        hideWhen: !main.isCSA && !main.isLateralTrainer && workoutScreen.container.currentViewName === "C360_VIEW"
        x:185 * main.hScaleFactor
        y:170 * main.vScaleFactor
        width:1000 * main.hScaleFactor
        dismissable: true
        dismissAnywhere: true
        isTransparent: true
    }

    Connections {
        target: machinecontrol
        onEstopChanged: {
            logger.logAlways("MachineControl: Safety Key " + (machinecontrol.estop ? "In" : "Out"))
        }
        onSafetyKeyQuickToggle: {
            miscStats.incrementUsage("Safety Key Toggle")
            logger.logAlways("MachineControl: Safety key quick toggle detected")
            logger.dumpLastLog()
        }
    }

    PopupManager{
        id: popupManager

//        safetyKeyOut: !machinecontrol.estop && (isTreadmill || isStepmill)
        safetyKeyOut: !machinecontrol.estop && isTreadmill
        // hide pop-ups while in diagnostic screen and other situations
        hidePopup: main.state === "DiagnosticState" || main.state === "SetupWizardState"  || main.restartRequired || options.ignoreFaults || main.firmwareUpdateRequired
        treadStopped: machinecontrol.treadmillSpeedActual < 1
        onEstopPopupTimedout: console.debug("estop timed out")

        Connections {
            target: machinecontrol

            onEstopChanged: {
                if(main.lowerSystem === "NUCLEUS" && estop === true && (isTreadmill || isStepmill)) {
                    popupManager.safetyKeyOut = false
                }
            }

            onSafetyKeyPulled: {
                if(main.lowerSystem === "NUCLEUS" && (isTreadmill || isStepmill)) {
                    popupManager.safetyKeyOut = true
                }
            }
        }
    }

    Connections {
        target: nucleusfirmwareloader
        onIsUpdatingChanged: errorManager.isNucleusFirmwareUpdating = nucleusfirmwareloader.isUpdating
    }

    ErrorManager{
        id:errorManager

        /**
         * @brief type:bool Property that determines if a fatal error has occured.
         */
        property bool fatalError: false

        /**
         * @brief type:bool Property that determines if an error occured that prevents the use of the machine.
         */
        property bool preventUse: false

        /**
         * @brief type:bool Property that determines if an error occured that results in a degraded use.
         */
        property bool degradedUse: false

        onErrorEvent: {
            if(options.ignoreFaults) {
                console.log("Ignoring error event: " + errorcode + " " + errormessage);
                return;
            }

            console.log("Error event: " + errorcode + " " + errormessage);
            logger.dumpLastLog();

            if(errorcode === "CN25") {
                main.firmwareUpdateRequired = true;
            }

            if(severity === ErrorManager.ERROR_FATAL) {
                fatalError = true;
            }

            if(severity >= ErrorManager.ERROR_PREVENTS_USE) {
                preventUse = true;
            } else if(severity >= ErrorManager.ERROR_DEGRADES_USE) {
                degradedUse = true;
            }

            if(!options.ignoreFaults) {
                var translatedErrorMessage = qsTranslate("ErrorManager", errormessage);
                popupManager.setErrorMessage(errorcode, translatedErrorMessage, severity);

                // briefly display new error at top of screen if pop-ups are hidden (as in diagnostic screens.)
                if(popupManager.hidePopup) {
                    infoBanner.showMessage(popupManager.currentMessage, 5);
                }

                if (errorcode === "X101" || errorcode === "X102") {
                    console.log("Stride Fault");
                    machinecontrol.strideFaulted = true;
                }
            }
        }

        onNucleusNonRecoverableFaultTriggered: {
            preventUse = true;
            popupManager.setNucleusFaultTriggeredMessage(msg);

            if(main.state === "RunState") {
                workoutScreen.stopWorkout();
            }

            if(popupManager.hidePopup) {
                infoBanner.showMessage(popupManager.currentMessage, 5);
            }
        }

        onNucleusRecoverableFaultTriggered: {
            preventUse = true;

            if(main.state === "RunState") {
                /* To Do: faults not implemented in Stepmill yet */
                if(!isStepmill)
                    workoutScreen.stopButtonPressedHandler();
            }
        }

        onNucleusFaultRecovery: {
            preventUse = true;
        }

        onNucleusFaultCleared: {
            preventUse = false;
        }

        onNucleusWarningTriggered: {
            infoBanner.showMessage(msg, 5);
        }

        onNucleusWarningCleared: {
            infoBanner.showMessage(msg, 5);
        }

        Component.onCompleted: {
            setNucleusObject(nucleus);
            setNucleusFaultInfoManager(nucleusFaultInfoMgr);
        }
    }

    StorageMgr{
        id:storageMgr
    }

    RemoteAccessDialog {
        id: accessDialog

        /**
         * @brief type:bool Property that determines if the remote access dialog is shown.
         */
        property bool showMe: main.state === "HomeState" && remoteAccess.isEnabled

        onShowMeChanged: {
            if (showMe) {
                show();
            } else {
                hide();
            }
        }
        isTransparent: true
    }

    RemoteAccess {
        id: remoteAccess
    }

    SoftwareUpdatePrompt {
        id: softwareUpdatePrompt

        /**
         * @brief type:bool Property that determines if the device is loaded.
         */
        property bool deviceLoaded: storageMgr.usbDeviceLoaded

        prompt: qsTr("Update Software Now?") + application.emptyString
        x: 300 * main.hScaleFactor
        y: 200 * main.vScaleFactor
        onNoClicked: hide()
        Component.onCompleted: hide()

        onDeviceLoadedChanged: {
            if (main.state != "HomeState") {
                return;
            }

            if (storageMgr.usbDeviceLoaded) {
                console.log("USB Device Loaded", storageMgr.usbDevicePath, storageMgr.usbBlockDevicePath);
                var hasPackages = storageMgr.checkForPackagesFile();
                console.log(hasPackages ? "Has Packages" : "No Packages");

                if (hasPackages) {
                    softwareUpdatePrompt.showPrompt();
                }
            } else {
                console.log("USB Device Removed", storageMgr.usbDevicePath, storageMgr.usbBlockDevicePath);

                if (softwareUpdatePrompt.visible) {
                    softwareUpdatePrompt.hide();
                }
            }
        }
    }

    Loader {
        id: screensaver
        z:1000
        anchors.fill: parent
        visible: false

        Text {
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.bottom: parent.bottom ; anchors.bottomMargin: 20 * main.vScaleFactor
            font.pointSize: 40
            color: "transparent"
            z:99
            text: qsTr("Touch to Begin") + application.emptyString
        }

        /**
         * @brief Starts the screen saver.
         */
        function start() {
            visible = true;
            screensaver.source = "Slideshow.qml";
            screensaver.item.zoomEffect = true;
            screensaver.item.fadeEffect = true;
            screensaver.item.slideInterval = 10000;
        }

        /**
         * @brief Stops the screen saver.
         */
        function stop() {
            visible = false;
            screensaver.source = "";

            if ((machineinfo.modelString === "----" && !setupWizard.isNucleus) || (setupWizard.isNucleus && !options.setupWizardCompleted)) {
                main.state = "SetupWizardState";
            } else {
                main.state = "HomeState";
            }
        }

        Connections {
            target: machinecontrol
            // wake up out of screen saver when pedaled.
            onPedalRPMChanged: {
                if(pedalRPM > 0) {
                    if(main.state === "ScreenSaverState" || main.state === "SleepModeState") {
                        screensaver.stop();
                    } else if (main.state === "HomeState") {
                        main.lcdBrightness = options.lcdBrightness;

                        if (!options.screenSaveTimeout && options.dimDisplayTimeout) {
                            dim_timer.restart();
                        }
                    }
                }

                if(workoutScreen.workoutActive && !(main.isTreadmill || main.isStepmill)) {
                    // update the pace profile with current pace during a workout.
                    aWorkoutRunData.setRPM(pedalRPM);
                    workoutProgram.profile2.setProperty(workoutProgram.profile2.currentSegment, "segmentMagnitude", pedalRPM);
                }
            }
            onDisplayWattsChanged: {
                aWorkoutRunData.setWatts(displayWatts);
                workoutManager.setWatts(displayWatts);
            }
            onStrideLengthTargetChanged: {
                workoutManager.strideLevel = strideLengthTarget;
            }
        }

        MouseArea {
            anchors.fill: parent
            onClicked: screensaver.stop()
        }
    }

    /**
     * @brief Starts a workout using the quick start workout.
     * @param type:string filename The quick start file to load/use.
     */
    function quickStart(filename) {
        if ((isTreadmill || isStepmill) && machinecontrol.treadmillSpeedActual !== 0) {
            return;
        }

        if (main.state === "SleepModeState") return;
        if (softwareUpdatePrompt.active) return;
        if (logUploadPrompt.visible) return;
        if (blockUseMessage.visible) return;
        if (accessDialog.visible) return;
        if (main.quickStartLoading) return; // prevent multiple consecutive loads

        if( main.state === "HomeState" || main.state == "ScreenSaverState" || main.state === "MenuState" ) {
            if(main.state === "ScreenSaverState") {
                screensaver.stop();
            }

            ipod_window.close();

            if(isTreadmill) {
                if(!filename) {
                    filename ="Tread_Quick_Start.xml";
                }
            }else if (isStepmill) {
                if(!filename) {
                    filename ="Step_Quick_Start.xml";
                }
            } else {
                if(!filename) {
                    filename = "Quick_Start.xml";
                }
            }

            main.quickStartLoading = true;
            main.channelSelected = "";
            print("Quickstart " + (filename ? filename : ""));

            workoutStats.incrementUsage(qsTr("Quick Start"));
            workoutProgram.loadWorkoutFileAsync("Program_Files/" + filename);
            aWorkoutRunData.programName = qsTr("QUICK START");
        }
    }

    /**
     * @brief Starts a given workout?
     * @param type:string item The workout item to load?
     */
    function startWorkout(item) {
        main.channelSelected = "";
        main.clearQLists();
        var programFile = item.program;
        workoutStats.incrementUsage(item.label);

        state = "LoadState";
        loadingProgramNotice.programItem = item;
        workoutProgram.loadWorkoutFileAsync(programFile);
    }

    Connections {
        target: workoutProgram
        onParsingFileChanged: {
            // saved programs are handled by the saved programs browser, so skip them
            if (!workoutProgram.parsingFile) {
                finishLoading(loadingProgramNotice.programItem);
            }
        }
    }

    /**
     * @brief Finishes loading a workout?
     * @param type:string The item to load?
     */
    function finishLoading(item) {
        if(!workoutProgram.isQuickStart) {
            mainProgItem.setProgramDetails(item);

            aWorkoutRunData.programName = item.label;
            main.state = "SetupState";
        } else {
            aWorkoutRunData.programName = qsTr("QUICK START");
            workoutStartManager.conditionalStart();
        }
    }

    /**
     * @brief Starts a workout conditionally?
     */
    function conditionalStart() {
        workoutManager.resetLevels();
        aWorkoutRunData.reset(main.isNewWorkoutSelected);
        workoutManager.setupWorkout();
        main.state = "RunState";
        workoutScreen.start();

        if(main.isTreadmill || main.isStepmill) {
            testwarning.show();
        } else if (main.isCSA && workoutProgram.isC360Program) {
            // Allow intro video to play
        } else {
            workoutManager.start();
        }
    }

    TreadmillWorkoutCountdownWarning {
        id:testwarning
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        onJumpStart: {
            workoutManager.jumpStart();
        }
        onTimedOut: {
            workoutManager.start();
            hide();
        }
        onVisibleChanged: {
            if(visible) {
                delayTimer.start();
            } else {
                countDown = 3; // reset counter when hiding dialog
            }
        }
        onTriggered: buzzer.warningBeep()
        Component.onCompleted: hide()

        Connections {
            target: workoutScreen
            /* Hide the dialog, which stops count, when workout active status
             * changes to false during count down.
             */
            onWorkoutActiveChanged: {
                if(testwarning.visible) {
                    testwarning.hide();
                }
            }

            onWorkoutStarted: {
                nucleus.workoutStart(workoutProgram.programIndex, workoutProgram.userWeight,
                                     workoutManager.workoutData.timeRemaining, workoutManager.workoutData.distanceRemaining*10,
                                     workoutManager.workoutData.calories)
                wahoo.startWorkout()
            }
            onWorkoutStopped: {
                nucleus.workoutStop();
                wahoo.stopWorkout()
                testwarning.hide()
            }
            onWorkoutPaused: {
                nucleus.workoutPause();
                wahoo.pauseWorkout()
                testwarning.hide()
            }
            onWorkoutResumed: {
                nucleus.workoutResume()
                wahoo.startWorkout()
            }
        }

        Timer{
            id: delayTimer
            interval: 350
            running: false
            repeat: false
            onTriggered: {
                testwarning.startTimeout(3);
                buzzer.warningBeep();
            }
        }
    }

    /**
     * @brief Clears all queue lists?
     */
    function clearQLists() {
        workoutManager.reset();
        mainProgItem.clearListView(); // Use to reset setup items
        aWorkoutRunData.reset(main.isNewWorkoutSelected);
    }

    Connections {
        target: machineinfo
        onModelStringChanged: {
            // Re-select workout menu
            workout_menu.selectWorkoutMenu()
            wahoo.equipmentType = machineinfo.productTypeString;
        }
    }

    ImageButton {
        id: screen_lock_button
        visible: false
        enabled: false
    }

    ScalableImage {
        id: background
        z: -2
        visible: main.state !== "SetupWizardState" && main.state !== "DiagnosticState"
        anchors.fill: parent
        sourceName: "StandardPage/View_Background"
    } // end Image background

    AttractVideo {
        id: videoRunner
        z: -2
        visible: main.state === "HomeState" && errorManager.preventUse == false && home_screen.buttonsEnabled === true && popupManager.estopShown === false
        anchors.fill: parent
    }

    Item {
        id: workoutStartManager

        /**
         * @brief type:bool Property that determines if the machine has been engaged such as a treadmill safety circuit?
         */
        property bool engaging: false

        /**
         * @brief Starts the workout conditionally?
         */
        function conditionalStart() {
//            if((main.isTreadmill || main.isStepmill)  && lowerSystem === "NUCLEUS") {
            if((main.isTreadmill)  && lowerSystem === "NUCLEUS") {
                if(engaging === false) {
                    engaging = true;
                    machinecontrol.safetyCircuitClose();
                } else {
                    main.conditionalStart();
                }
            } else {
                main.conditionalStart();
            }
        }

        Connections {
            target: machinecontrol
            onSafetyCircuitEngaged: {
                if(workoutStartManager.engaging === true) {
                    workoutStartManager.engaging = false;
                    main.conditionalStart();
                }
            }
            onSafetyCircuitDisengaged: {
                if(workoutStartManager.engaging === true) {
                    workoutStartManager.engaging = false;
                    main.quickStartLoading = false;
                }
            }
            onSafetyKeyInserted: {
                if(workoutStartManager.engaging === false) {
                    workoutStartManager.engaging = true;
                }
            }
        }
    }

    Item {
        id: diagnoticScreenEntry
        anchors.fill: parent
        visible: (popupManager.estopShown === true || errorManager.preventUse === true || blockUseMessage.visible === true) && main.state === "HomeState"
        property bool flag: false

        Timer {
            id: diagnoticScreenEntryTimeout
            interval: 5000
            onTriggered: diagnoticScreenEntry.flag = false
        }

        Rectangle {
            id: upperLeftCorner
            color: "transparent"
            height: 200
            width: height
            anchors.top: parent.top
            anchors.left: parent.left
            MouseArea {
                anchors.fill: parent
                onPressAndHold: {
                    buzzer.secretBeep(0)
                    diagnoticScreenEntry.flag = true
                    diagnoticScreenEntryTimeout.start()
                }
            }
        }

        Rectangle {
            id: lowerRightCorner
            color: "transparent"
            height: 200
            width: height
            anchors.bottom: parent.bottom
            anchors.right: parent.right

            MouseArea {
                enabled: diagnoticScreenEntry.flag
                anchors.fill: parent
                onPressAndHold: {
                    buzzer.secretBeep(0)
                    accessKeyPad.visible = true
                    diagnoticScreenEntryTimeout.stop()
                }

            }
        }
    }

    Loader {
        id: home_screen_loader
        sourceComponent: touch_home_screen_component
        anchors.fill: parent
        anchors.rightMargin: 0
        anchors.bottomMargin: 0
        anchors.leftMargin: 0
        anchors.topMargin: 0
        z: 4
        opacity: 0
        enabled: (visible && opacity > 0.0)
        Component.onCompleted: {
            item.updateLanguageIcon(options.defaultLanguage);
        }

        Component {
            id: touch_home_screen_component

            HomeScreen {
                id: touch_home_screen
                x: 0
                y: 0
                opacity: 1
                visible: !application.crashRecovery && !popupManager.estopShown && !errorManager.preventUse && !blockUseMessage.visible && !accessDialog.visible
                /**********************************************************************
                  Truth table for buttonsEnabled
                  +-------------------+-------------------+------------++----------------+
                  | productTypeString | treadIsCalibrated | standalone || buttonsEnabled |
                  +-------------------+-------------------+------------++----------------+
                  |         ""        |     don't care    |   false    ||     false      |
                  |       "Bike"      |     don't care    |   false    ||     true       |
                  |    "Elliptical"   |     don't care    |   false    ||     true       |
                  |    "Treadmill"    |        true       |   false    ||     true       |
                  |    "Treadmill"    |        false      |   false    ||     false      |
                  |    "Stepmill"     |     don't care    |   false    ||     true       |
                  |         ""        |     don't care    |    true    ||     false      |
                  |       "Bike"      |     don't care    |    true    ||     true       |
                  |    "Elliptical"   |     don't care    |    true    ||     true       |
                  |    "Treadmill"    |        true       |    true    ||     true       |
                  |    "Treadmill"    |        false      |    true    ||     true       |
                  |    "Stepmill"     |     don't care    |    true    ||     true       |
                  +-------------------+-------------------+------------++----------------+
                 **********************************************************************/
                buttonsEnabled: !(((machineinfo.productTypeString === "") && (options.ignoreFaults === false)) ||
                    (main.isTreadmill && (machinecontrol.isCalibrated == false) && (options.ignoreFaults == false)) ||
                    ((machineinfo.productTypeString === "") && (options.ignoreFaults == true)))
                                || options.manufacturingTestModeEnabled === true
                showCardio360: machineinfo.productType === WorkoutProgram.ELLIPTICAL || machineinfo.productTypeString === "Lateral Trainer"
                showMonumentsButton: main.isAlpineRunner
                onQuickstartPressed: {
                    quickStart();
                }
                onC360QuickstartPressed: {
                    if (main.isLateralTrainer) {
                        main.quickStart("Cardio_360_Quick_Start_LateralTrainer.xml")
                    } else {
                        main.quickStart("Cardio_360_Quick_Start.xml")
                    }
                }
                onMonumentsPressed: {
                    main.state = "MenuState";
                    workout_menu.selectMenu("MONUMENTS")
                }
                onLoginPressed: main.state = "LoginState"
                onRegisterPressed: main.state = "RegisterState"
                onSignoutPressed: {
                    main.state = "HomeState";
                }
            }
        }
    }

    // Display the workout program image while program loads from XML
    Item {
        id: loadingProgramNotice
        anchors.fill: parent
        opacity: 0
        enabled: (visible && opacity > 0.0)

        /**
         * @brief type:variant Property containing the items for a workout program.
         */
        property variant programItem : undefined

        TextLabel {
            id: loadingProgramLabel
            text: loadingProgramNotice.programItem ? qsTranslate("Menu", loadingProgramNotice.programItem.label) : ""
            font.pointSize: 14 * main.vScaleFactor
            anchors {
                horizontalCenter: parent.horizontalCenter
                verticalCenter: parent.verticalCenter
            }
        }

        ProgressBar {
            id: progressBar
            height: 40 * main.vScaleFactor
            width: 360 * main.hScaleFactor
            value: workoutProgram.parsingProgress
            anchors {
                horizontalCenter: parent.horizontalCenter
                top: loadingProgramLabel.bottom
                topMargin: 10 * main.vScaleFactor
            }
            opacity: 0.5
        }
    }

    Loader {
        id: workout_menu_loader
        sourceComponent: touch_workout_menu_component
        anchors.fill: parent
        focus: main.state === "MenuState"
        opacity: 0
        enabled: (visible && opacity > 0.0)

        Component {
            id: touch_workout_menu_component

            WorkoutFinder {
                id: touch_workout_menu
                anchors.fill: parent
                onWorkoutSelected: {
                    startWorkout(item);
                }
            }
        }
    }

    AudioManager {
        id: audioManager
        debug: true
        tvPlaying: tv.playing && !tv.isIpodInput
        ipodPlaying: false
        ipodPlayingVideo: false
        browserPlaying: browserAudioWatcher.soundIsPlaying
        btconnected: bluetoothAdapter.connected
        onAudioSourcesChanged: print("audio sources: " + audioSources)
    }

    Process {
        id: bluetoothDeviceAudioWatcher

        /**
         * @brief type:bool Property that determines if sound is playing from the web browser.
        */
        property bool available: false

        program: "scripts/watch_btdev_audio.sh"
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim();
            console.log("bluetoothDeviceAudioWatcher stdout", data);

            if (data === "yes") {
                bluetoothDeviceAudioWatcher.available = true;
            } else {
                bluetoothDeviceAudioWatcher.available = false;
            }
        }
        onFinished: {
            console.log("bluetoothDeviceAudioWatcher finished");
            bluetoothDeviceAudioWatcher.available = false;
        }
    }

    Process {
        id: browserAudioWatcher

        /**
         * @brief type:bool Property that determines if sound is playing from the web browser.
        */
        property bool soundIsPlaying: false

        program: "scripts/watch_browser.sh"
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim();
            console.log("browserAudioWatcher stdout", data);

            if (data === "yes") {
                browserAudioWatcher.soundIsPlaying = true;
            } else {
                browserAudioWatcher.soundIsPlaying = false;
            }
        }
        onFinished: {
            console.log("browserAudioWatcher finished");
            browserAudioWatcher.soundIsPlaying = false;
        }
    }

    Process {
        id: browserMute
        program: "scripts/browsermute.sh"

        property bool muted: false
        property bool _flag: false

        /**
                 * @brief Mutes the sound.
                 * @param type:string Flags for muting sound?
                 */
        function setMute(flag) {
            _flag = flag
            browserMute.arguments = [flag];
            browserMute.start();
        }
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim();
            console.log("output from browsermute:", data);
            muted = _flag
        }
    }

    Process {
        id: vlcMute
        program: "scripts/vlcmute.sh"

        /**
         * @brief Mutes the sound.
         * @param type:string Flags for muting sound?
         */
        function setMute(flag) {
            vlcMute.arguments = [flag]
            vlcMute.start()
        }
        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim();
            console.log("output from vlcMute:", data);
        }
    }

    Connections {
        target: audioManager
        onVideoSourceChanged: {
            console.log("Video Source is: " + audioManager.videoSource);
        }
        onTvVideoAllowedChanged: {
            muteTimer.stop();
            muteTimer.start();
        }
        onVirtActiveVideoAllowedChanged: {
            muteTimer.stop();
            muteTimer.start();
        }
    }

    Timer {
        id: muteTimer
        running: false
        interval: 1000
        onTriggered: {
            running = false
            if(audioManager.tvVideoAllowed || audioManager.virtActiveVideoAllowed || audioManager.ipodVideoAllowed || (audioManager.ipodAllowed && bluetoothAdapter.connectedHeadset)) {
                console.log("Unmuting VLC");
                vlcMute.setMute(0);
            } else {
                console.log("Muting VLC");
                vlcMute.setMute(1);
            }
        }
    }

    Connections {
        target: audioManager
        onBtAllowedChanged: {
            if (audioManager.btAllowed) {
                btMute.setMute(0);
            } else {
                btMute.setMute(1);
            }
        }
    }

    Process {
        id: btMute
        program: "scripts/btmute.sh"

        /**
         * @brief Mutes the sound.
         * @param type:string Flags for muting sound?
         */
        function setMute(flag) {
            btMute.arguments = [flag];
            btMute.start();
        }

        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim();
            console.log("output from btmute:", data);
        }
    }

    Television {
        id: tv
        onClicked: {
            buzzer.buttonPress()
            tv.fullscreen = tv.fullscreen ? false: true
        }
        muted: false
        verbose: false

        onUsbResetFailedChanged: {
            if(usbResetFailed)
                main.rebootRequired = true
        }

        Connections {
            target: audioManager
            onTvVideoAllowedChanged: {
                if(audioManager.tvVideoAllowed) {
                    tv.tuneChannelID(tv.currentChannel);
                    tv.on = true;
                } else {
                    tv.stop();
                }
            }

            onTvAllowedChanged: {
                tv.muted = !audioManager.tvAllowed;
                print("tv muted:" + audioManager.tvAllowed);
            }
        }
    }

    ModalDialog {
        id: rebootMessage

        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter

        blockKeys: true
        property bool showPopup: main.state === "HomeState" && main.rebootRequired && tv.usbResetFailed
        onShowPopupChanged: {
            if (showPopup){
                show()
                rebootMessage.startTimeout(5)
            } else
                hide()
        }

        onTimedOut: {
            main.startReboot()
            hide()
        }

        onVisibleChanged: {
            if(!visible) {
                // reset counter when hiding dialog
                rebootMessage.countDown = 5
            }
        }
        onTriggered: buzzer.warningBeep()
        Component.onCompleted: hide()

        Item {
            anchors.fill: parent
            z: parent.z + 1

            TextLabel {
                id: warningText2

                width: 0.8 * parent.width
                height: 99 * main.vScaleFactor

                text: qsTr("TV Error Detected.") + "\n" + qsTr(" Console Rebooting in ...") + application.emptyString
                anchors.horizontalCenter: parent.horizontalCenter
                anchors.top: parent.top
                anchors.topMargin: 35 * main.vScaleFactor
                horizontalAlignment: Text.AlignHCenter
                wrapMode: Text.WordWrap
                font.pixelSize: 40 * main.vScaleFactor
            }

            NumberLabel {
                // show count down to timeout of prompt (started by startTimeout(seconds) method of ModalDialog superclass
                id: countdownText2

                // display count down (with a leading 0 if remaining seconds < 10
                text: ":" + ((rebootMessage.countDown.toString().length < 2) ? "0" : "") + rebootMessage.countDown
                font.pixelSize: 96 * main.vScaleFactor

                anchors.horizontalCenter: parent.horizontalCenter
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 73 * main.vScaleFactor
            }
        }
    }

    function startReboot () {
        if(tv.usbResetFailed) {
//            tvTestHud.usbhubcount++
            console.log("**************************Calling rebootConsole.start() with arguments" + rebootConsole.arguments)
            console.log("TV Error Stats:  Channel Change Count: " + tvTestHud.count +  ", USB Errors Detected: " + tvTestHud.usbcount + ", VLC Error state: " + tv.vlcErrorOccurred)
            rebootConsole.start()
        }
    }

    Process { id: rebootConsole
        program: "scripts/rebootConsole.sh"

        onStarted: {
            console.log("Reboot console  started")
        }

        onReadyReadStandardOutput: {
            var output = String(readAllStandardOutput())
            output = output.trim()
            console.log( "rebootConsole process (STDOUT):" + output)
        }

        onReadyReadStandardError: {
            var output = String(readAllStandardError())
            output = output.trim()
            console.log( "rebootConsole process (STDERR):" + output)
        }
    }


    /**
     * @brief type:alias Property that contains the manager for the SAP mode.
     */
    property alias sapManager: sapLoader.item

    Loader {
        id: sapLoader
        sourceComponent: sapManagerComponent

        /**
         * @brief Reloads the SAP manager.
         */
        function reload() {
            sapLoader.sourceComponent = undefined;
            sapLoader.sourceComponent = sapManagerComponent;
        }

        Component {
            id: sapManagerComponent

            Item {
                /**
                 * @brief type:alias Property that contains the manager for the SAP manager?
                 */
                property alias item: manager

                SapManager {
                    id: manager
                    onChannelFound: {
                    }
                }
            }
        }
    }

    Timer {
        running: true; interval: 3000; repeat: true
        onTriggered: {
        }
    }

    TvRecordingManager {
        id: tvRecordingManager
    }

    TvRecordingMessage {
        id: tvRecordingMessage
        visible: tvRecordingManager.recording || tvRecordingManager.copying
        z: 9999999
    }

    // Full-screen Video Frame, for TV or video workouts, with data overlay
    Rectangle {
        id: fullscreenVideoFrame

        /**
         * @brief type:QtObject Property that contains the currently hosted fullscreen player.
         */
        property QtObject videoItem  // the currently hosted fullscreen player

        /**
         * @brief Displays the video frame in full screen mode.
         * @param type:item item The item passed here must have a "fullscreen" property
         */
        function showFullscreen(item) {
            item.fullscreen = true;
            fullscreenSurface.player = item;
            fullscreenSurface.aspect = item.aspect;
            fullscreenVideoFrame.visible = true;
            fullscreenVideoFrame.videoItem = item;
        }

        /**
         * @brief End full screen mode.
         */
        function endFullscreen() {
            if(fullscreenVideoFrame.videoItem) {
                fullscreenVideoFrame.videoItem.fullscreen = false;
                fullscreenVideoFrame.videoItem = null;
                fullscreenSurface.player = null;
            }

            fullscreenVideoFrame.visible = false;
        }

        anchors.fill: parent
        visible: fullscreenVideoFrame.videoItem != null
        color: "black"
        z: 97

        VLCSurface {
            id: fullscreenSurface
            playing: fullscreenVideoFrame.visible
            anchors.fill: parent
            DiagnosticOverlay {
                id: diagOverlay
            }
        }

        MouseArea {
            // This mouse area is masked by the VideoDataOverlay when shown,
            // so click screen to minimize is only available in a non-workout fullscreen TV mode
            onClicked: {
                fullscreenVideoFrame.endFullscreen();
                buzzer.buttonPress()
            }
            anchors.fill: parent
        }

        VideoDataOverlay {
            id: videoOverlay
            visible: main.state === "RunState"
            isTreadmill: main.isTreadmill
//            showHR: heartRateManager.heartRateAvailable
//            heartRate: heartRateManager.heartRate
            incline: workoutManager.treadInclineGUI
            speed: workoutScreen.unitState == "ENGLISH"  || main.isStepmill ? workoutManager.treadSpeedGUI : workoutManager.treadSpeedMetricGUI.toFixed(1)
            speedUnits: (main.isStepmill ? qsTr("SPM") : workoutScreen.unitState === "ENGLISH" ? qsTr("MPH") : qsTr("KPH") ) + application.emptyString
            rpm: machinecontrol.pedalRPM
            level: workoutManager.workloadLevel

            opacity: 0.6
            enabled: (visible && opacity > 0.0)
            anchors.fill: parent
            showData: (main.state === "RunState")
        }

        Column {
            spacing: 0 * main.vScaleFactor
            anchors {
                left: parent.left
                leftMargin: 0 * main.hScaleFactor
                bottom: parent.bottom
                bottomMargin: 0
            }
            visible: true//fullscreenVideoFrame.visible
            z: fullscreenSurface.z + 1

           /* Item {
                id: stopButton
                width: 53 * main.hScaleFactor
                height: 45 * main.vScaleFactor

                ScalableImage {
                    anchors.fill: parent
                    anchors.margins: 12 * main.scaleFactor
                    fillMode: Image.PreserveAspectFit
                    sourceName: "Stop_icon_WHITE";
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        root.stop()
                        buzzer.buttonPress()
                    }

                    Rectangle {
                        anchors.fill: parent
                        color: main.colorGenericStimulusArea
                        opacity: 0.3
                        visible: main.showStimulusArea
                    }
                }
            }*/

            Item {
                id: sizeButton
                width: 150 * main.hScaleFactor
                height: 150 * main.vScaleFactor

                ScalableImage {
                    width: 40 * main.hScaleFactor
                    height: 32 * main.vScaleFactor
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.verticalCenter: parent.verticalCenter
                    sourceName: "UpperToolBar/expand_icon";
                }

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        fullscreenVideoFrame.endFullscreen(tv.player)
                        buzzer.buttonPress()
                    }

                    Rectangle {
                        anchors.fill: parent
                        color: main.colorGenericStimulusArea
                        opacity: 0.3
                        visible: main.showStimulusArea
                    }
                }
            }
        }
    }

    LanguageMenu {
        id: language_menu
        opacity: 0
        enabled: (visible && opacity > 0.0)
        z:99
        anchors.fill: parent
        focus: main.state === "LanguageMenuState"
        onLanguageSelected: {
            if(lang) {
                application.selectLanguage(lang);
                home_screen.updateLanguageIcon(lang);
            }

            main.state = "HomeState";
        }
    }

    // return to default language after a period of inactivity in the home screen
    Timer {
        id: lang_timer
        running: main.state === "HomeState"
        interval: 5 * 60 * 1000
        onTriggered: {
            if (main.state === "HomeState") {
                application.selectLanguage(options.defaultLanguage); // reset language after workout
                home_screen.updateLanguageIcon(options.defaultLanguage);
            }
        }
    }
    Item {
        id: tvTestHud

        property alias channelChangeTime: channelChangeTimeSlider.value
        property int usbFaultTime: options.triggerUSBFaultInterval
        property int usbcount: 0
//        property int usbhubcount: 0
        property int count: 0
        property ListModel tuneTimeModel: tv.tuneTimes

        opacity: 0.5
        visible: enabled
        enabled: options.autoChannelChangeInterval > 99
        width: childrenRect.width
        height: childrenRect.height
        anchors { right: parent.right; rightMargin: 25 * main.hScaleFactor; verticalCenter: parent.verticalCenter }
        z:99
        Rectangle {
            id: tvTestItemRect
            color:"green"
            width: childrenRect.width
            height: childrenRect.height

            Column {
                spacing: 3 * main.vScaleFactor
                Text { text: "Time: " + (tv.tuneTimes.count > 0 ? tv.tuneTimes.get(tv.tuneTimes.count-1).time/1000.0  + "s" : "") }
                Row {
                    id: barGraph
                    height: 50 * main.vScaleFactor
                    width: 140 * main.hScaleFactor
                    spacing: 1 * main.hScaleFactor
                    Repeater {
                        model: tvTestHud.tuneTimeModel
                        delegate: Component {
                            Rectangle {
                                height: barGraph.height * (model.time / 5000.0)
                                width: barGraph.width / tvTestHud.tuneTimeModel.numberTimesToStore
                                anchors { bottom: parent ? parent.bottom : undefined }
                            }
                        }
                    }
                }

                Text { visible: true; text: qsTr("Interval: ") + (channelChangeTimeSlider.off ? qsTr("OFF") : tvTestHud.channelChangeTime) }
                Text { visible: true; text: qsTr("Count: ") + tvTestHud.count }

                SliderControl{
                    id: channelChangeTimeSlider
                    property bool off: channelChangeTimeSlider.value === 0 //channelChangeTimeSlider.maxValue
                    value: options.autoChannelChangeInterval //maxValue
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: 0.8 * parent.width
                    height: 30 * main.vScaleFactor
                    minValue: 0
                    maxValue: 600 * 1000
                    stepValue: 1000
                    showValue: false
                    onValueChanged: {options.autoChannelChangeInterval = value}
                }

                Text { text: "USB Fault Trigger Interval: " + tvTestHud.usbFaultTime/1000.0  + "s" }
                Text { text: "USB Faults Triggered: " + tvTestHud.usbcount }
                Text { text: "USB Faults Detected: " + tv.numUSBErrors }
//                Text { text: "USB Hub Resets Needed: " + tvTestHud.usbhubcount }
                Text { text: "USB Resetting: " + tv.usbReseting }
                Text { text: "USB Reset Failed: " + tv.usbResetFailed }
                Text { text: "VLC Error Occured: " + tv.vlcErrorOccurred }
            }
        }
    }

    Item {
        id: debugOverlay

        /**
         * @brief type:string Property that contains all supported audio sources.
         */
        property string asources: ((audioManager.ipodPlaying ? "ip " : "") +
                                  (audioManager.ipodPlayingVideo ? "iv " : "") +
                                  (audioManager.virtActivePlaying ? "va " : "") +
                                  (audioManager.tvPlaying ? "tv " : "") +
                                  (audioManager.browserPlaying ? "web " : "") +
                                  (audioManager.btconnected ? "bt " : ""))

        /**
         * @brief type:string Property that contains all allowed audio devices.
         */
        property string aallowed: ((audioManager.ipodAllowed ? "ip " : "") +
                                  (audioManager.ipodVideoAllowed ? "iv " : "") +
                                  (audioManager.btAllowed ? "bt ": "") +
                                  (audioManager.virtActiveAllowed ? "va " : "") +
                                  (audioManager.virtActiveVideoAllowed ? "vv " : "") +
                                  (audioManager.tvAllowed ? "tv " : "") +
                                  (audioManager.tvVideoAllowed ? "tvv " : "") +
                                  (audioManager.browserAllowed ? "web " : ""))

        opacity: 0.5
        visible: false
        x: 20 * main.hScaleFactor
        y: 480 * main.vScaleFactor
        z:99

        Rectangle {
            color:"gray"
            width: childrenRect.width
            height: childrenRect.height

            Column {
                spacing: 3
                Text { visible: true;           text: qsTr("audio:") + debugOverlay.asources }
                Text { visible: true;           text: qsTr("allowed:") + debugOverlay.aallowed }
                Text { visible: true;           text: "mux:"+ audioManager.currentMuxSource }
                Text { visible: isTreadmill; 	text: qsTr("target incline:") + machinecontrol.treadmillInclineTarget / 10.0 }
                Text { visible: isTreadmill; 	text: qsTr("actual incline:") + machinecontrol.treadmillInclinePercentGradeActual / 10.0 }
                Text { visible: isTreadmill || isStepmill; 	text: qsTr("target speed:") + machinecontrol.treadmillSpeedTarget / 10.0 }
                Text { visible: isTreadmill || isStepmill; 	text: qsTr("actual speed:") + machinecontrol.treadmillSpeedActual / 10.0 }
                Text { visible: !isTreadmill && !isStepmill; 	text: qsTr("Workload:") + machinecontrol.workload }
                Text { visible: true;           text: qsTr("time warp:") + timeFactorControl.value + "x" }

                SliderControl {
                    id: timeFactorControl
                    value: main.timeFactor
                    width: 150 * main.hScaleFactor
                    minValue: 0
                    maxValue: 30
                    showValue: false
                    onValueChanged: main.timeFactor = value

                    thumbImage.sourceName: "Components/Red_Dot";
                }
            }
        }
    }

    Rectangle {
        id: languageDebug
        visible: false
        x: 20 * main.hScaleFactor
        y: 300 * main.vScaleFactor
        color: "#80659d32"
        width: 200 * main.hScaleFactor
        height: childrenRect.height
        z:99999999999999999999

        Column {
            spacing: 2
            Repeater {
                model: [
                    { lang: "en", caption: qsTranslate("LanguageMenu", "ENGLISH") + application.emptyString } ,
                    { lang: "ru", caption: qsTranslate("LanguageMenu", "RUSSIAN") + application.emptyString } ,
                    { lang: "fr", caption: qsTranslate("LanguageMenu", "FRENCH") + application.emptyString } ,
                    { lang: "zh", caption: qsTranslate("LanguageMenu", "CHINESE") + application.emptyString } ,
                    { lang: "de", caption: qsTranslate("LanguageMenu", "GERMAN") + application.emptyString } ,
                    { lang: "nl", caption: qsTranslate("LanguageMenu", "DUTCH") + application.emptyString } ,
                    { lang: "ar", caption: qsTranslate("LanguageMenu", "ARABIC") + application.emptyString } ,
                    { lang: "es", caption: qsTranslate("LanguageMenu", "SPANISH") + application.emptyString } ,
                    { lang: "pt", caption: qsTranslate("LanguageMenu", "PORTUGUESE") + application.emptyString } ,
                    { lang: "ko", caption: qsTranslate("LanguageMenu", "KOREAN") + application.emptyString } ,
                    { lang: "ja", caption: qsTranslate("LanguageMenu", "JAPANESE") + application.emptyString } ,
                    { lang: "it", caption: qsTranslate("LanguageMenu", "ITALIAN") + application.emptyString } ,
                ]
                delegate: Component {
                    Text {
                        font { pixelSize: 16 * main.vScaleFactor}
                        font.bold: application.currentLanguage === modelData.lang
                        color: main.colorFontLight
                        text: modelData.caption

                        MouseArea {
                            anchors.fill: parent
                            onClicked: {
                                buzzer.buttonPress()
                                var lang = modelData.lang
                                application.selectLanguage(lang)
//                                home_screen.updateLanguageIcon(lang)
                            }
                            Rectangle {
                                anchors.fill: parent
                                color: main.colorGenericStimulusArea
                                opacity: 0.3
                                visible: main.showStimulusArea
                            }
                        }
                    }
                }
            }
        }
    }

    Rectangle {
        id: wahooDebug
        visible: false

        x: 20 * main.hScaleFactor
        y: 300 * main.vScaleFactor
        color: "#80659d32"
        width: 200 * main.hScaleFactor
        height: childrenRect.height
        z:99999999999999999999

        Column {
            spacing: 2

            Text {
                text: "Wahoo Advertised Name: "
            }

            Text {
                text: wahoo.advertisedName
            }

            Text {
                text: "Wahoo Connected: " + wahoo.connected
            }

            Text {
                text: "Wahoo Heart Rate: " + wahoo.gemHeartRate
            }
        }
    }

    // Keys to control the simulator
    Keys.onPressed: {
        switch(event.key){
            case Qt.Key_Q: { // fake stride-up for testing
                keymapper.keyPressHandler(8);
                break;
            }
            case Qt.Key_A: { // fake stride-down for testing
                keymapper.keyPressHandler(3);
                break;
            }
            case Qt.Key_R: { // Ctrl+r allows recording a TV stream
                if (event.modifiers & Qt.ControlModifier) {
                    console.log("ctrl-r pressed");

                    if (!tvRecordingManager.recording) {
                        tvRecordingManager.startRecording();
                    }
                }

                break;
            }
            case Qt.Key_S: { // toggle on/off simulation of random pedaling / heart rate
                main.loginAvailable = !main.loginAvailable;
//                pedalSim.simulating = !pedalSim.simulating;
                break;
            }
            case Qt.Key_G: {  // toggle holding heartrate grips
//                pedalSim.holdingGrips = !pedalSim.holdingGrips;
                break;
            }
            // must have simulation turned off to use the following controls
            case Qt.Key_BracketRight: { // ']' increase heartrate
//                pedalSim.heartRate++;
                break;
            }
            case Qt.Key_BracketLeft: { // '[' decrease heartrate
//                pedalSim.heartRate--;
                break;
            }
            case Qt.Key_Period: { // '.' increase pedal speed
//                pedalSim.pedalRPM++;
                break;
            }
            case Qt.Key_Comma: { // ',' decrease pedal speed
//                pedalSim.pedalRPM--;
                break;
            }
            case Qt.Key_AsciiTilde: // '~' toggle debug HUD
                debugOverlay.visible = !debugOverlay.visible;
                break;
            case Qt.Key_QuoteDbl: {
                languageDebug.visible = !languageDebug.visible;
                break;
            }
            case Qt.Key_C: // 'C' toggle show mouse cursor
                application.showCursor = !application.showCursor;
                break;
            case Qt.Key_O: // 0 key also takes screen shot
            case Qt.Key_P: {// 'P' take screen shot (photo)
                buzzer.buttonPress();
                print("taking screenshot " + main.screenShotCount);
                application.screenShot("/media/usb/screenshot_" + main.screenShotCount + ".png");
                infoBanner.showMessage(qsTr("Screenshot ") + main.screenShotCount + qsTr(" saved."), 3);
                main.screenShotCount++;
                break;
            }
            case Qt.Key_K: { // toggle safety key simulation
//                pedalSim.safetyKeyIn = !pedalSim.safetyKeyIn;
//                print("estop:" + pedalSim.safetyKeyIn);
                break;
            }
            case Qt.Key_D: { // quick diagnostic entry
                accessKeyPad.visible = true;
                break;
            }
            case Qt.Key_T: {
                if (event.modifiers & Qt.ShiftModifier) {
                  tvTestHud.visible = !tvTestHud.visible
                } 
                break
            }
            case Qt.Key_W: {
                wahooDebug.visible = !wahooDebug.visible;
                break;
            }
        }
    }

    Keys.onReleased: {
        switch (event.key) {
            case Qt.Key_Q: { // fake stride-up for testing
                keymapper.keyReleaseHandler(62);
                break;
            }
            case Qt.Key_A: { // fake stride-down for testing
                keymapper.keyReleaseHandler(63);
                break;
            }
        }
    }

    Connections {
        target: options
        onSimulationModeChanged: {
            console.debug("Simulation mode changed: " + options.simulationMode)
        }
    }

    Connections {
        target: heartrate
        onTelemetryHeartRateChanged: heartRateManager.setHeartRate("telemetry", telemetryHeartRate)
        onContactHeartRateChanged: heartRateManager.setHeartRate("contact", contactHeartRate)
    }

    RecoveryManager{
        id: recoveryManager
        onCompleted: {
            errorManager.preventUse = false
            errorManager.degradedUse = false
            popupManager.reset()
        }
    }

    Connections{
        target: errorManager
        onErrorEvent:{
            recoveryManager.errorTriggeredSlot(errorcode)
        }

        onConsoleDetectedFaultRecovered: {
            errorManager.preventUse = false
            errorManager.degradedUse = false
            popupManager.reset()
        }
    }

    function personPresentSupported()
    {
        return ( machineinfo.modelString === "TI1000" ||
                machineinfo.modelString === "TC900" ||
                machineinfo.modelString === "TC650" ||
                machineinfo.modelString === "TC400" ||
                machineinfo.modelString === "VC900")
    }

    PersonPresentDetectionManager{
        id:personPresentDetectionManager
        isEnabled: (options.ppdEnabled) && personPresentSupported()
    }

    KeyMapper{
        id: keymapper
        function handleStart() {
            if ((isTreadmill || isStepmill) && machinecontrol.treadmillSpeedActual !== 0) {
                return
            }

            if((isTreadmill || isStepmill) && canStartWorkout == false) {
                return
            }

            if (blockUseMessage.visible || errorManager.fatalError || errorManager.preventUse || main.state === "SleepModeState")
                return
            if (accessDialog.visible)
                return
            if (softwareUpdatePrompt.active)
                return
            if (logUploadPrompt.visible)
                return
            if (bluetoothAdapter.connecting)
                return

            quickStart()
            main.channelSelected = ""
        }
        onStartQuickStartPressed: {
            keymapper.handleStart()
        }
        onStartPressed: {
            keymapper.handleStart()
        }
    }

    WorkoutRunData {
       id: aWorkoutRunData

        workoutProgram: workoutProgram.programIndex
        productTypeString: machineinfo.productTypeString
        heartRate: heartRateManager.heartRate
        workloadLevel: workoutManager.workloadLevel
        paceLevel: workoutManager.paceLevel
        wattsAt60: workoutManager.wattsAt60;
        timeFactor: main.timeFactor
        strideLength: workoutManager.strideLevel
    }

    WorkoutManager {
        id: workoutManager

        property bool withLimit: true

        productType: machineinfo.productType
        workoutData: aWorkoutRunData
        userData: userData
        heartRate: heartRateManager.heartRate
        treadSpeedGUI: isStepmill ? machinecontrol.stepmillSpeedTarget : machinecontrol.treadmillSpeedTarget
        treadInclineGUI: machinecontrol.treadmillInclineTarget

        onWattsAt60Changed: {machinecontrol.workload = wattsAt60; aWorkoutRunData.setWatts(wattsAt60)}
        onSpeedChanged: {
            if(isTreadmill)
                machinecontrol.treadmillSpeedTarget = (main.state == "RunState") ? treadSpeed : 0 // allow non-zero speed only in RunState
            else if(isStepmill)
                machinecontrol.stepmillSpeedTarget = (main.state == "RunState") ? treadSpeed : 0 // allow non-zero speed only in RunState
        }
        onInclineChanged: {
            machinecontrol.treadmillInclineTarget = treadIncline
            if(main.lowerSystem === "NUCLEUS") {
                workoutData.incline = treadIncline / 10.0
            }
        }

        onStrideLevelChanged: machinecontrol.strideLengthTarget = strideLevel

        workoutProgram: workoutProgram

        onWorkoutDataChanged: {
            // TODO Send workout data from console to Nucleus for CSAFE purposes
        }

        timeFactor: main.timeFactor
        cooldownDuration: options.cooldownTime
        //treadInclineMax: configurationData.maxInclinePercentGrade
        //treadInclineMin: configurationData.minInclinePercentGrade

        onWorkoutModeChanged:
        {
            var modeStr = ""
            switch(mode) {
            case WorkoutRunData.WARM_UP_MODE: modeStr = "Warm Up"; break;
            case WorkoutRunData.COOL_DOWN_MODE: modeStr = "Cool Down"; break;
            case WorkoutRunData.WORKOUT_MODE: modeStr = "Workout"; break;
            case WorkoutRunData.SUMMARY_MODE: modeStr = "Summary"; break;
            }

            print("WorkoutManager: Workout Mode is " + modeStr)
            aWorkoutRunData.setWorkoutMode(mode);
            if (mode == WorkoutRunData.SUMMARY_MODE) {
                workoutScreen.stopWorkout()
                treadSpeed = 0
                machinecontrol.treadmillSpeedTarget = treadSpeed
            }
            if (mode == WorkoutRunData.COOL_DOWN_MODE) {
                workoutScreen.showMessage(qsTr("COOL DOWN") + application.emptyString)
            }
        }
        onDisplayHRPrompt: workoutScreen.displayHRPrompt(index);
    } // end WorkoutManager

    MessagePopup {
        id: speedInclineLimitedPopup
        blockKeys: false
        isTransparent: true
        Component.onCompleted: hide()
    }

    Connections {
        target: machinecontrol
        onSpeedLimited: {
            workoutManager.treadSpeed = machinecontrol.treadmillSpeedTarget
            var msg = qsTr("Speed limited to ") + application.emptyString

            var speed = workoutManager.treadSpeed / 10.0
            if (workoutScreen.unitState === "ENGLISH") {
                msg += speed + " " + qsTr("MPH") + application.emptyString
            } else {
                msg += Globals.mileToKm(speed).toFixed(1) + " " + qsTr("KPH") + application.emptyString
            }

            speedInclineLimitedPopup.showMessage(msg, 3)
        }
        onInclineLimited: {
            workoutManager.treadIncline = machinecontrol.treadmillInclineTarget
            var msg = qsTr("Incline limited to ") + workoutManager.treadIncline / 10 + " %";
            speedInclineLimitedPopup.showMessage(msg, 3)
        }
        onInclineInhibited: {
            workoutManager.treadIncline = machinecontrol.treadmillInclineTarget
            infoBanner.showMessage("Incline disabled", 5)
        }
    }

    ProgSetup {
        id: mainProgItem
        enabled: (visible && opacity > 0.0)

        focus: main.state === "SetupState"

        onStartProgram:{
            workoutProgram.setupComplete(aWorkoutRunData.totalTime, aWorkoutRunData.totalDistance)
            workoutScreen.updateAdjustModel()
            workoutStartManager.conditionalStart()
//            workoutManager.start();
//            main.state = "RunState";
//            workoutScreen.start();
        }

        onSaveCustomProgram: workoutProgram.saveProgram(programFileName,saveLocation);

        anchors.fill: parent
        opacity: 0
        z: 8
    }

    WorkoutProgram {
        id: workoutProgram

        userData: userData
        userWeightMax: {
            if(main.lowerSystem === "NUCLEUS") {
                return nucleus.maxUserWeightLbs
            } else {
                return 400
            }
        }
        metricUnits: options.metricUnits
        saveWorkoutEnabled: options.saveWorkoutEnabled
        property int maxSegments: Math.min( profile1.count, profile2.count)
        inclineTable: machineinfo.inclineTable
        speedTable: machineinfo.speedTable
        speedTableMetric: machineinfo.speedTableMetric
        brakeWattsTable: machineinfo.brakeWattsTable
        paceTable: machineinfo.paceTable
        brainboardProductTypeString: machineinfo.productTypeString
        isChangeToNewWorkout: main.isNewWorkoutSelected
    }

    BlueToothLowEnergy {
        id: ble
    }

    WorkoutScreen {
        id: workoutScreen
        enabled: (visible && opacity > 0.0)

        visible: main.state === "RunState" && !fullscreenVideoFrame.visible

        onWorkoutStopped: {
            browserAudioWatcher.kill()
            audioManager.reset()
            main.state = "WorkoutSummaryState";
            workoutManager.stop();
            stopPrompt.hide()
            viewContainerBanner.hide()
            if (bluetoothAdapter.discovering) {
                bluetoothAdapter.stopDiscovery()
            }

            workout_summary.setData( aWorkoutRunData )
            //prepare data for blutooth advertisment
            ble.exportWorkoutDataForBleno(aWorkoutRunData,application)

            // add to statistics
            statisticManager.mileage += aWorkoutRunData.totalDistance * 100
            statisticManager.runHours += aWorkoutRunData.totalTime
            statisticManager.totalDistance += aWorkoutRunData.totalDistance * 100
            statisticManager.writeFile()

            // end of workout
            workoutManager.reset();
            workoutManager.resetLevels();
            aWorkoutRunData.mode = WorkoutRunData.SUMMARY_MODE
            mainProgItem.clearListView();

            workoutScreen.container.destroyViews()

            if(!main.isNewWorkoutSelected) {
                audioManager.setMute(false)
                tv.currentChannel = -1

                // if bluetooth is connected, disconnect the bluetooth device
                if(bluetoothAdapter.connected){
                    bluetoothAdapter.disconnect()
                }
                if (bluetoothAdapter.connectedHeadset)
                    bluetoothAdapter.disconnectHeadset()
            }
            machinecontrol.autoStrideEnabled = false

            if(options.triggerWorkoutAutoRestart) {
                console.log("Restarting autostart timer................................................................................. autostart in " + autostartTimer.interval + "ms")
                autostartTimer.restart()
            }
        }//

        onWorkoutPaused: {
            workoutManager.pause()
        }
        onWorkoutStarted: {
            quickStartLoading = false
            if (options.cabEnabled) {
                var firstChannel = tv.channelList.getChannelItem(0)
                if (firstChannel) {
                    console.log("CAB: Autotuning to", firstChannel.id)
                    tv.tuneChannelID(firstChannel.id)
                }
            }

            if(options.triggerWorkoutAutoRestart) {
                console.log("Restarting autostop timer......................................................................................autostop in "  + autostopTimer.interval +"ms.")
                autostopTimer.restart()
            }
        }


        anchors.fill: parent
        opacity: 0


    }

    WorkoutSummary {
        id: workout_summary
        timeLeft: idle_timer.secondsLeft()
        opacity: 0
        enabled: (visible && opacity > 0.0)
        focus: main.state === "WorkoutSummaryState"
        onClose: {
            saveHasHappened = false
            main.state = "HomeState"
        }
        onSave: {
            var success = aWorkoutRunData.saveResults()
            if(success) {
                saveHasHappened = true
                infoPopup.showMessage(qsTr("Summary has been saved.") + application.emptyString, 5);
            }
            else
                infoPopup.showMessage(qsTr("Error. Check your memory device.") + application.emptyString, 5);
        }
    }

    SetupWizard{
        id: setupWizard

        focus: main.state === "SetupWizardState"
        isNucleus: main.lowerSystem === "NUCLEUS" || main.lowerSystem === "NUCBOOT"
        nucleusUpdateAvailable: nucleusfirmwareloader.isUpdateAvailable

        opacity: 0
        enabled: (visible && opacity > 0.0)
        onSetupWizardComplete: {
            main.restartRequired = true
            main.state = "HomeState"
        }
        onLanguageSelected:{
            if(lang){
                application.selectLanguage(lang)
                home_screen.updateLanguageIcon(lang)
                options.defaultLanguage = lang
            }
        }

    }

    BlueToothHeartRate {
        id: bluetoothLEAdapter

        onScanningChanged: {
            if(scanning === true) {
                heartRateManager.currentBtAddress = ""
                heartRateManager.clearBluetoothHeartRateDeviceList()
                if (bluetoothLEAdapter.nextHrAddress !== "") {
                    bluetoothLEAdapter.getHrServiceHandle(bluetoothLEAdapter.nextHrAddress, bluetoothLEAdapter.nextHrAddressType)
                    bluetoothLEAdapter.nextHrAddress = ""
                    bluetoothLEAdapter.nextHrAddressType = ""
                }
            }
        }

        onHeartrateChanged: {
            bluetoothLEAdapter.connecting=false
            heartRateManager.setHeartRate("bluetooth", bluetoothLEAdapter.heartrate)
        }
        //onScanningChanged: {
            //if(bluetoothLEAdapter.scanning == true)

        //}
        onHrTimeout: {
            console.log("Heartrate timed out...now disconnecting")
            console.debug("Heartrate timed out...now disconnecting")

            bluetoothLEAdapter.disconnect(bluetoothLEAdapter.deviceMAC,bluetoothLEAdapter.hrHandle)
        }
        onConnected: {
            heartRateManager.currentBtAddress = bluetoothLEAdapter.deviceMAC

            /* Close media browser if showing on HR connected */
            if(main.state === "HomeState") {
                if(ipod_window === true) {
                    main.showMediaBrowser = false
                }
            }
        }
        onHrDisconnectingComplete: {
            heartRateManager.currentBtAddress = ""
            heartRateManager.clearBluetoothHeartRateDeviceList()
            if (bluetoothLEAdapter.nextHrAddress !== "") {
                bluetoothLEAdapter.getHrServiceHandle(bluetoothLEAdapter.nextHrAddress, bluetoothLEAdapter.nextHrAddressType)
                bluetoothLEAdapter.nextHrAddress = ""
                bluetoothLEAdapter.nextHrAddressType = ""
            }
        }
    }


    HeartRateManager{
        id: heartRateManager
        onBluetoothHRConnect: {
            if(heartRateManager.currentBtAddress !== address) {
                if (bluetoothLEAdapter.hrConnected) {
                    // Disconnect them and store address for later reconnect
                    bluetoothLEAdapter.nextHrAddress = address
                    bluetoothLEAdapter.nextHrAddressType = addressType
                    bluetoothLEAdapter.disconnect(bluetoothLEAdapter.deviceMAC, bluetoothLEAdapter.hrHandle)
                    return
                }

                bluetoothLEAdapter.getHrServiceHandle(address, addressType)
            }
        }
    }

    Connections{
        target: workoutScreen
        onWorkoutStopped: heartRateManager.reset()

    }

    // Both SetupWizard and DiagnosticsMain use this, so put it here
    ModalDialog {
        id:productSelectedConfirmation
        property alias text: dialogText.text

        Text {
            id: dialogText
            anchors.fill: parent
            anchors.verticalCenter: parent.verticalCenter
            color: main.colorFontLight
            anchors.horizontalCenter: parent.horizontalCenter
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            wrapMode: Text.WordWrap
            font.bold: false
            font.pixelSize: 50 * main.hScaleFactor
            z: parent.z + 1

        }
        Component.onCompleted: hide()
        onTimedOut: hide()
    }

    MediaBrowser {
        id: ipod_window

        allowFullscreen: false
        z: 90
        opacity: visible ? 1.0 : 0.0
        enabled: visible
        visible: main.showMediaBrowser

        isDialogVisible: visible

        onClose: {
            main.showMediaBrowser = false
        }
        onVisibleChanged: {
            if(visible)
                shown()
            else
                hidden()
        }
        anchors {   horizontalCenter: parent.horizontalCenter
                    top: parent.top
                    topMargin: 140 * main.vScaleFactor
        }

        Behavior on opacity { NumberAnimation { duration: 200; easing.type: Easing.InOutQuad } }
    }

    DiagnosticsMain {
        id: diagnosticsMain
        opacity: 0
        enabled: (visible && opacity > 0.0)
        anchors.fill: parent

        Connections {
            target: diagnosticsMain
            ignoreUnknownSignals: true

            onClose: {
                main.state = "HomeState"
                if(machinecontrol.isCalibrated){
                    machinecontrol.treadmillInclineTarget = 0
                }
            }
        }
    }

    Keypad {
        id: accessKeyPad
        length: 5
        show_entry: true
        instructions: qsTr("ENTER PASSCODE") + application.emptyString
        textEntryMinWidth: 300 * main.hScaleFactor
        textEntryHeight: 42 * main.vScaleFactor
        z: 200

        anchors.centerIn: parent
        width: 417 * main.hScaleFactor
        height: 436 * main.vScaleFactor
        radius: main.defaultRadius
        border.color: main.colorUIBorder
        border.width: main.widthBorder

        opacity: 1
        visible: false

        onClose: {
            diagnosticsMain.diagnosticAccessCode = 0
            accessKeyPad.visible = false
            accessKeyPad.clear()
        }

        onValueEntered: {
            if(val.length == 5) {
                diagnosticsMain.diagnosticAccessCode = Number(val)
                accessKeyPad.visible = false
                if (diagnosticsMain.accessLevel >= 0)
                    main.state = "DiagnosticState"
            }
        }

        // re-translate the strings when the language changes
        Connections {
            target: application
            onLanguageChanged: {
                accessKeyPad.instructions = qsTr("ENTER PASSCODE") + application.emptyString
            }
        }
    }

    MouseArea {
        // Capture clicks outside keypad to dismiss it
        anchors.fill: accessKeyPad
        enabled: accessKeyPad.visible
        z: accessKeyPad.z - 1
        onClicked: {}
    }

    MouseArea {
        // Capture clicks outside keypad to dismiss it
        anchors.fill: parent
        enabled: accessKeyPad.visible
        z: accessKeyPad.z - 2
        onClicked: accessKeyPad.close()
    }

    LogUploader {
        id: logUploader
    }

    LogUploadPrompt {
        id: logUploadPrompt
    }


    Options {
        id: options
        objectName: "options"
        onContactHRPriorityChanged: {
            heartRateManager.setContactPriority(contactHRPriority)
        }
        onLcdBrightnessChanged: main.lcdBrightness = lcdBrightness
        onCabEnabledChanged: {
            nucleus.cabSetEnable(cabEnabled)
        }
        onIgnoreFaultsChanged: {
            if (options.ignoreFaults) {
                popupManager.reset()
            }
        }
        onLoggingEnabledChanged: {
            logger.enabled = loggingEnabled
        }
        onDisableBuzzerChanged: {
            buzzer.setBuzzerEnabled(!options.disableBuzzer)
        }
    }

    property int lcdBrightness: options.lcdBrightness
    onLcdBrightnessChanged: {
        main.setLcdBrightness(lcdBrightness)
    }

    function setLcdBrightness(percent) {
         lcdBrightnessProcess.setBrightness(percent)
    }

    Process {
        id: lcdBrightnessProcess
        program: "scripts/set_brightness.sh"

        function setBrightness(percent) {
            lcdBrightnessProcess.arguments = [ (percent*6000/100).toFixed(0) ]
            lcdBrightnessProcess.start()
        }
    }

    Binding {
        target: application
        property: "showCursor" ; value: options.showCursor
    }

    // Idle timer
    // Time out interval is set by state transition table below.
    // The timer is reset by a screen touch (event sent by application object.)
    // Timeout action is defined in onTriggered event.  Many screens return to HomeScreen state.
    Timer {
        id: idle_timer

        repeat: true
        running: true
        interval: 10000
        onRunningChanged: idleSecondsTimer.reset()
        onIntervalChanged: idleSecondsTimer.reset()

        function secondsLeft() {
            var sec = (interval / 1000) - idleSecondsTimer.seconds
            if(sec < 0 )
                return 0
            else
                return sec
        }

        onTriggered: {
            switch(main.state) {
                case "HomeState":{  // idle timeout on home-screen goes to screen-saver or sleep
                    if(options.screenSaveTimeout) {  // screensaver enabled
                        main.state = "ScreenSaverState"
                        screensaver.start()
                    } // otherwise, go to sleep mode if enabled
                    else if(options.sleepModeTimeout) {
                        main.state = "SleepModeState"
                    }

                    if (remoteAccess.isEnabled) {
                        // Exit screensaver immediately
                        screensaver.stop()
                    }

                    // disconnect bluetooth device if connected
                    if (bluetoothAdapter.connected) {
                        bluetoothAdapter.disconnect()
                    }
                    if (bluetoothAdapter.connectedHeadset)
                        bluetoothAdapter.disconnectHeadset()

                    if(!options.screenSaveTimeout && !options.sleepModeTimeout && options.erpEnabled){
                        erpSleep()
                    }
                    break;
                }
                case "LoginState":
                case "RegisterState": {
                    main.state = "HomeState";
                    break;
                }
                case "WorkoutSummaryState": {
                    workout_summary.close() // This will make sure the bt advertise dialog closes on timeou
                    break
                }
                case "MenuState":
                case "SetupState":
                case "LanguageMenuState": {
                    // disconnect bluetooth device if connected
                    if (bluetoothAdapter.connected) {
                        bluetoothAdapter.disconnect()
                    }
                    if (bluetoothAdapter.connectedHeadset)
                        bluetoothAdapter.disconnectHeadset()
                    main.state = "HomeState";
                    break;
                }
                case "ScreenSaverState": {
                    if (options.sleepModeTimeout) {
                        main.state = "SleepModeState"
                    } else if(options.erpEnabled){
                        erpSleep()
                    }
                    break;
                }
                case "SetupWizardState":
                {
                    break;
                }
                case "SleepModeState":
                {
                    if(options.erpEnabled){
                        erpSleep()
                    }

                    break;
                }
            } // end switch
        } // end onTriggered
    }

    Timer {
        id: dim_timer
        interval: options.dimDisplayTimeout * 1000
        running: false
        repeat: false
        onTriggered: {
            if (main.state != "HomeState" && main.state != "ScreenSaverState")
                return

            // Set to 40% of brightness setting
            var brightnessLevel = parseInt(0.40 * options.lcdBrightness)
            main.lcdBrightness = brightnessLevel
            console.log("Dim timer fired. Set brightness to", brightnessLevel)
        }
    }

    Timer {
        id:idleSecondsTimer

        property int seconds: 0

        function reset() {
            idleSecondsTimer.seconds = 0
            idleSecondsTimer.restart()
        }

        interval: 1000
        running: true
        onRunningChanged: seconds = 0
        repeat: true
        onTriggered: seconds++
    }

    Timer {
        id: playBluetooth
        interval: 1000; repeat: false; running: false

        onTriggered: {
            if (bluetoothAdapter.currentConnectedDevice) {
                if (bluetoothAdapter.currentConnectedDevice.player) {
                    bluetoothAdapter.currentConnectedDevice.player.play()
                }
            }
        }
    }

    TrueBlueAdapter{
        id: bluetoothAdapter

        onConnectedChanged: {
            if (connected) {
                btMute.setMute(0)
                audioManager.setVolume(audioManager.volumeDefault)
                playBluetooth.start()
            }
        }

        onConnectedHeadsetChanged: {
            if (tv.isIpodInput) {
                // restart tv if iPod is currently playing, so that it will start streaming the ipod
                // input to pulseaudio, which is required for the headset to function
                tv.mrl = ""
                tv.tuneChannelID("IPODINPUT")
            }
        }
    }
    // Expire bluetooth connection after 2 minutes without starting a workout
    Timer {
        id: bluetoothSessionTimer
        running: (main.state !== "RunState") && bluetoothAdapter.connected
        interval: 2 * 60 * 1000
        onTriggered: {
            ipod_window.close()
            if (bluetoothAdapter.connected)
                bluetoothAdapter.disconnect()
            if (bluetoothAdapter.connectedHeadset)
                bluetoothAdapter.disconnectHeadset()
        }
    }

    // Expire bluetooth Heartrate connection after 5 minutes without starting a workout
    Timer {
        id: bluetoothLESessionTimer
        running: (main.state !== "RunState") && bluetoothLEAdapter.hrConnected
        interval: 30 * 1000
        onTriggered: {
            if (bluetoothLEAdapter.hrConnected)
                console.debug("main HR timeout")
                bluetoothLEAdapter.disconnect()
        }
    }

    Rectangle {
        id: touchSplat

        property int size: 10
        opacity: 0.0 // Hide until first touch
        width: size
        height: width
        radius: width
        z: 100

        transform: Translate { x:-touchSplat.size/2 ; y: -touchSplat.size/2 }

        color: main.colorUIBorder //"#0099FF"

        ParallelAnimation {
            id:splat
            NumberAnimation { target: touchSplat; properties: "size" ; from: 15 ; to: 0; duration: 500; easing.type: Easing.InOutQuad }
            NumberAnimation { target: touchSplat; properties: "opacity" ; from: 0.5 ; to: 0; duration: 500; easing.type: Easing.InOutQuad }
        }
    }

    Connections {
        // reset the idle timer when there's a screen touch
        target: application
        onScreenTouched: {
            splat.stop() // Stop in-progress splat
            idle_timer.restart()
            lang_timer.restart()
            touchSplat.x = x
            touchSplat.y = y
            touchSplat.opacity = 0.5
            touchSplat.size = 15
            if(main.state == "SleepModeState")
                main.state = "HomeState"
            else if (main.state === "HomeState") {
                main.lcdBrightness = options.lcdBrightness
                if (!options.screenSaveTimeout && options.dimDisplayTimeout)
                    dim_timer.restart()
            }
        }
        onMouseReleased: {
            touchSplat.x = x
            touchSplat.y = y
            splat.restart()
        }
    }

    Connections {
        target: machineinfo
        onDetectionComplete: {
            main.lowerSystem = lowerSystem
            if(lowerSystem === "NO_LOWERSYSTEM" && !options.ignoreFaults) {
                blockUseMessage.hide()
                errorManager.errorReceived(ErrorManager.ERRORNO_CONSOLE_USBCONNECTIONTERMINATED,statisticManager.totalDistance)
            }

            if (main.lowerSystem === "NUCLEUS" || main.lowerSystem === "NUCBOOT") {
                if (!options.setupWizardCompleted) {
                    main.state = "SetupWizardState"
                }
            }

        }
    }

    onStateChanged: {
        print("Main: UI state = " + main.state)

        if(state === "HomeState"){
            videoRunner.start()
            if(machineinfo.productType === WorkoutProgram.ELLIPTICAL) {
                machinecontrol.workload = 0
            }
        }
    }

    Process {
        id: paVolControl
        program: "scripts/true-pa-volume-control.sh"
        property bool isMute
        property int _currentVolSetting: 0

        function setVolume(vol_setting){

            /* Only run the script if the vol setting is different */
            if(_currentVolSetting === vol_setting) {
                return
            }

            _currentVolSetting = vol_setting

            var vol = (6554 * vol_setting) + 6550
            var volStr = vol.toString();
            paVolControl.arguments = ["vol", volStr]
            paVolControl.start()
        }

        function mute() {
            paVolControl.arguments = ["mute"]
            paVolControl.start()
        }

        function info() {
            paVolControl.arguments = ["info"]
            paVolControl.start()
        }

        function parseOutput(str){
            var strList = str.split(" ")
            if(strList[0] === "info"){
                var volumeInt = parseInt(strList[2])
                var isMuteStr = strList[3]

                if(isMuteStr === "yes") {
                    isMute = true
                } else if(isMuteStr === "no") {
                    isMute = false
                }

                console.debug(volumeInt + " " + isMuteStr)
            }
        }

        onReadyReadStandardOutput: {
            var data = String(readAllStandardOutput()).trim()
            console.log("output from paVolControl:",data)
            parseOutput(data)
        }
    }

    Timer {
        id: check_nucleus_faults_timer
        interval:24*3600*1000
        running: true
        repeat: true
        onTriggered: {
            // Retrieve Nucleus error logs
            console.log("Retrieving Nucleus fault log")
            errorManager.retrieveFaultLogs()
        }
    }

    states: [
        State {
            name: "HomeState"
            PropertyChanges {target: home_screen_loader ; opacity: 1 }
            PropertyChanges {target: screen_lock_button ; opacity: 1 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false; isNewWorkoutSelected: false }
            // set timeout for screen-saver or dim mode or sleep mode
            PropertyChanges {target: idle_timer ;
                interval: {
                    if(options.screenSaveTimeout)
                        return options.screenSaveTimeout*1000
                    else if(options.sleepModeTimeout)
                        return options.sleepModeTimeout*1000
                    else if(options.erpEnabled && options.erpTimeout)
                        return options.erpTimeout*1000
                    else
                        return 0
                }
            }
            PropertyChanges { target: dim_timer; running: options.dimDisplayTimeout > 0 && !options.screenSaveTimeout }
            PropertyChanges {target: workout_summary ; dialogVisible: false }
            PropertyChanges {target: main; showMediaBrowser: false}
        },
        State {
            name: "LoginState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 60000}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
        },
        State {
            name: "RegisterState"
            PropertyChanges { target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 180000}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
        },
        State {
            name: "LanguageMenuState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 1 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutMenuTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
        },

        State {
            name: "MenuState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 1 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutMenuTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
        },
        State {
            name: "LoadState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: loadingProgramNotice ; opacity: 1 }
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: main; showMediaBrowser: false}
        },
        State {
            name: "SetupState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 1 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutSetupTimeout * 1000}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
        },
        State {
            name: "RunState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 1 }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            // turn off idle timer - run screen will have its own
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 0}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
        },
        State {
            name: "WorkoutSummaryState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0 }
            PropertyChanges {target: workout_summary ; opacity: 1 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: options.workoutSummaryTimeout * 1000 }
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}

        },
        State {
            name: "DiagnosticState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 1 ; z: 50}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0  }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 0 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 300000}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
        },
        State {
            name: "SetupWizardState"
            PropertyChanges {target: home_screen_loader ; opacity: 0 }
            PropertyChanges {target: language_menu; opacity: 0 }
            PropertyChanges {target: workout_menu_loader ; opacity: 0 }
            PropertyChanges {target: diagnosticsMain ; opacity: 0 ; z: 0}
            PropertyChanges {target: mainProgItem ; opacity: 0 }
            PropertyChanges {target: workoutScreen ; opacity: 0  }
            PropertyChanges {target: workout_summary ; opacity: 0 }
            PropertyChanges {target: setupWizard ; opacity: 1 }
            // set timeout to return to main screen
            PropertyChanges {target: main; lcdBrightness: options.lcdBrightness; restoreEntryValues: false }
            PropertyChanges {target: idle_timer ; interval: 0}
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
        },
        State {
            name: "ScreenSaverState"
             // set timeout until moving from screensaver to dim-mode, sleep-mode, or power-down
            PropertyChanges {target: idle_timer ; interval: options.sleepModeTimeout ? options.sleepModeTimeout * 1000 : options.erpTimeout * 1000 }
            PropertyChanges { target: dim_timer; running: options.dimDisplayTimeout > 0 }
            PropertyChanges {target: main; showMediaBrowser: false}
        },
        State {
            name: "SleepModeState"
             // set timeout until moving from screensaver to power-down/sleep mode
            PropertyChanges {target: main; lcdBrightness: 0; restoreEntryValues: true ; explicit: true }
            PropertyChanges {target: idle_timer ; interval: options.erpTimeout * 1000 }
            PropertyChanges {target: dim_timer; running: false }
            PropertyChanges {target: main; showMediaBrowser: false}
        }

    ] // end states
    transitions: [
        Transition {
                NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
        },

        Transition {
            from: "HomeState"
            to: "MenuState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    workout_menu.reset()
                }
            }
        },
        Transition {
            from: "MenuState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    home_screen.updateLanguageIcon(options.defaultLanguage)
                    clearQLists()
                    main.quickStartLoading = false
                }
            }
        },
        Transition {
            from: "SetupState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    home_screen.updateLanguageIcon(options.defaultLanguage)
                    clearQLists()
                    main.quickStartLoading = false
                }
            }
        },
        Transition {
            from: "DiagnosticState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    home_screen.updateLanguageIcon(options.defaultLanguage)
                    clearQLists()
                    main.quickStartLoading = false
                    syncDisk.start()
                }
            }
        },
        Transition {
            from: "HomeState"
            to: "DiagnosticState"
            ScriptAction {
                script: {
                    videoRunner.stop()
                    softwareUpdatePrompt.hide()
                }
            }
        },
        Transition {
            from: "HomeState"
            to: "LoginState"
            NumberAnimation { target: signin_window; properties: "opacity"; easing.type: Easing.Linear; duration: 100 }
            NumberAnimation { target: home_screen; properties: "opacity"; easing.type: Easing.Linear; duration: 100 }
        },
        Transition {
            from: "WorkoutSummaryState"
            to: "HomeState"
            NumberAnimation { property: "opacity" ; duration: 200; easing.type: Easing.InOutQuad }
            ScriptAction {
                script: {
                    workout_summary.saveHasHappened = false
                    application.selectLanguage(options.defaultLanguage) // reset language after workout
                    home_screen.updateLanguageIcon(options.defaultLanguage)
                    clearQLists()
                    main.quickStartLoading = false
                }
            }
        },
        Transition {
            to: "RunState"
            ScriptAction {
                script: {
                    videoRunner.stop()
                }
            }
        }
    ]
}
