import WorkoutRunData 1.0
import WorkoutManager 1.0
import TouchAppEnumerations 1.0
import QtQuick 2.7
import QtQuick.Layouts 1.3
import "../string_format.js" as Format
import Components 1.0
import "../"

Item {
    id: root

    // REQUIRED property: for translation to work (via qsTranslate), it must know where the external model is stored.
    // Without this you will likely see blank strings in your data table.
    property string sourceFile: "DataSummaryTable"

    property int screenHeight: parent.height
    property int screenWidth: parent.width

    property int _expandedHeight: screenHeight * 854 / 1080
    property int _expandedWidth:  screenWidth * 1734 / 1920

    property int _collapsedHeight: screenHeight * 104 / 1080
    property int _collapsedWidth: screenWidth * 1417 / 1920

    property bool isExpanded: false
    property int expandTime: 500

    property alias dataDisplayModel: dataSelectionTable.dataDisplayModel

    state: ""

    property alias selectedCount: dataSelectionTable.selectedCount

    property string colorBackground: isExpanded ? main.colorDialogBackground : main.colorToolbarBackground

    property string distanceLabel: workoutScreen.unitState === "ENGLISH" ? QT_TR_NOOP("MILES") : QT_TR_NOOP("KM")
    property string paceLabel: workoutScreen.unitState === "ENGLISH" ? QT_TR_NOOP("PACE/MILE") : QT_TR_NOOP("PACE/KM")
    property string distanceLabelTranslated: qsTr(distanceLabel) + application.emptyString
    property string paceLabelTranslated: qsTr(paceLabel) + application.emptyString

    // re-translate the strings when the language changes
    Connections {
        target: application
        onLanguageChanged: {
           root.distanceLabelTranslated = qsTr(root.distanceLabel) + application.emptyString
            root.paceLabelTranslated = qsTr(root.paceLabel) + application.emptyString
        }
    }

    Connections {
        target: workoutScreen
        onUnitStateChanged: {
            root.distanceLabel = workoutScreen.unitState === "ENGLISH" ? qsTr("MILES") : qsTr("KM")
            root.paceLabel = workoutScreen.unitState === "ENGLISH" ? qsTr("PACE/MILE") : qsTr("PACE/KM")
            root.distanceLabelTranslated = qsTr(root.distanceLabel)
            root.paceLabelTranslated = qsTr(root.paceLabel)
        }
    }

    width: _collapsedWidth
    height: _collapsedHeight
    clip: true

    function updateDataDisplayItem(item) {
        var i;
        var obj;
        var menu = dataDisplayModel;

        for(i = 0; i < menu.count; i++) {
            obj = menu.get(i);

            if(obj.label === item.label && obj.listName === item.listName) {
                obj.value = item.value;
                return;
            }
        }
    }

    /**
     * Simply have two states: collapsed and expanded
     * Use the appropriate height for each state
     */
    states: [
        State {
            name: ""
            when: root.isExpanded === false
            PropertyChanges {
                target: root
                height: _collapsedHeight
                width: _collapsedWidth
            }
        },
        State {
            name: "expanded"
            when: root.isExpanded === true
            PropertyChanges {
                target: root
                height: _expandedHeight
                width: _expandedWidth
            }
        }
    ]
    /**
     * Animations when changing states
     */
    transitions: [
        Transition {
            ParallelAnimation {
                PropertyAnimation {
                    target: root
                    property: "height"
                    duration: expandTime
                }
                PropertyAnimation {
                    target: root
                    property: "width"
                    duration: expandTime
                }
            }
        }
    ]

    function reset() {
        dataSelectionTable.reset();
        isExpanded = false;
    }

    Rectangle {
        id: background
        opacity: 1
        color: colorBackground
        anchors.fill: parent
        radius: isExpanded ? 0 : main.defaultRadius
        border.color: isExpanded ? main.colorUIBorder : "transparent"
        border.width: main.widthBorder
    }

    RowLayout {
        id: topDisplay
        width: _collapsedWidth
        height: _collapsedHeight
        spacing: 0
        anchors.top: parent.top
        anchors.horizontalCenter: parent.horizontalCenter

        Rectangle {
            Layout.preferredHeight: 18 * main.vScaleFactor
            Layout.preferredWidth: 91 * main.hScaleFactor
            Layout.alignment: Qt.AlignRight | Qt.AlignVCenter
            color: "transparent"

            StandardIcon {
                id: expandButton
                width: 25 * main.hScaleFactor
                height: 18 * main.vScaleFactor
                anchors.right: parent.right

                onClicked: root.isExpanded = !root.isExpanded

                sourceName: root.isExpanded ? "UpperToolBar/collapse_icon" : "UpperToolBar/expand_icon";
            }
        }

        Item {
            id: dataDisplayArea
            Layout.preferredHeight: 0.74 * _collapsedHeight
            Layout.fillWidth: true
            Layout.alignment: Qt.AlignHCenter | Qt.AlignVCenter

            Grid {
                id: dataGrid
                columns: selectedCount
                rows: 1
                anchors.fill: parent

                Repeater {
                    model: dataDisplayModel
                    delegate: displayItem
                }
            }

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    buzzer.buttonPress()
                    root.isExpanded = !root.isExpanded
                }
            }
        }
    }

    Component {
        id: displayItem

        Column {
            height: dataDisplayArea.height
            width: dataDisplayArea.width / dataGrid.columns
            spacing: 0

            NumberLabel {
                id: value
                height: parent.height * 0.75
                width: parent.width
                visible: true
                font.pixelSize: height
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignTop
                text: model.value
            }

            TextLabel {
                id: label
                height: parent.height * 0.25
                width: parent.width
                font.pixelSize: height
                isHairline: true
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignBottom
                text: model.label === "DISTANCE" ? distanceLabelTranslated : model.label === "PACE" ? paceLabelTranslated  : qsTranslate(root.sourceFile, model.label) + application.emptyString
                wrapMode: Text.NoWrap
            }

            BeatingHeart {
                id: beatingHeartIcon
                anchors.verticalCenter: parent.verticalCenter
                anchors.right: parent.right
                anchors.rightMargin: 0.05 * parent.width
                visible: model.isSelected && model.isHR && heartRateManager.heartRateAvailable
            }
        }
    }

    Text {
        id: dataDisplayInstructionText
        width: parent.width
        height: 30 * main.vScaleFactor
        anchors.top: topDisplay.bottom
        anchors.topMargin: 10
        color: "lightgray"
        font.pixelSize: 11 * main.vScaleFactor
        font.bold: false
        horizontalAlignment: Text.AlignHCenter
        text: qsTranslate("WorkoutDataDisplay", "Select or deselect data to customize your workout display") + application.emptyString
        wrapMode: Text.WordWrap
        visible: isExpanded
    }

    Item {
        id: dataSelectionArea
        width: 0.827 * parent.width
        anchors.top: dataDisplayInstructionText.bottom
        anchors.topMargin: 20 * main.vScaleFactor
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.bottom: parent.bottom
        anchors.bottomMargin: 20 * main.vScaleFactor

        DataSummaryTable {
            id: dataSelectionTable
            selectable: true
            tableWidth: width - 2*tableHMargin
            anchors.fill: parent
        }
    }
}
